/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   marcel@mulch-online.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GLOBALS_H
#define GLOBALS_H
 
#ifdef HAVE_CONFIG_H
 #include <config.h>
#else
 #define VERSION "1.0 release candidate" 
#endif

/** Global constants, enums and defines for moneypenny
*/
 
#define APPLICATION_NAME "MoneyPenny"

/**
* The next section describes the default path and file 
* names of all HoBIT / Moneypenny-Files
*/
#include <qstring.h>

void setMP_DIR(char * argv0); // to be called from main.cpp, when debugging
QString MP_DIR(); //returns moneypenny's main directory as an absolute path

//Installation directories
#define MONEYPENNY_HELP_DIR "help" 
#define MONEYPENNY_ICON_DIR "icons" 
#define MONEYPENNY_BANKDATA_DIR "bankdata" 

//Writable directories, final paths are set in guiFindMedium
#define MONEYPENNY_HBCIDATA_DIR "hbcidata"
#define MONEYPENNY_MAIN_DIR "moneypenny_data"

//Filenames
#define MONEYPENNY_OPENHBCI_FILE "openhbci.conf"
#define MONEYPENNY_AQMONEY_FILE "moneypenny.conf"
#define MONEYPENNY_TEMPLATE_FILE "templates.conf"
#define MONEYPENNY_TRANSLATIONS_DIR "translations"
#define MONEYPENNY_ENCRYPTED_CONF_FILE "moneypenny.bf"
#define MONEYPENNY_PASSWORD_FILE "pwmd.dat"
#define MONEYPENNY_ALL_BANKDATA_FILE "bankdata.txt"
#define MONEYPENNY_HBCI_BANKDATA_FILE "fints_institute.csv"
#define MONEYPENNY_KNOPPIX_MOUNTPOINT "/mnt/usb_moneypenny"



// SQL Connection
#define MYSQL_DRIVER 		 "QMYSQL3"  /** MySQL Driver */
#define MYSQL_DATABASE	 	 ":winestore:"
#define MYSQL_USER 		 "root"
#define MYSQL_PASSWORD	 	 ""
#define MYSQL_HOST		 "localhost"
#define USE_MYSQL_FEATURES 	 false 




/* region settings , 280 is Germany*/
#define MONEYPENNY_DEFAULT_COUNTRY 280   

/* default name of the export file*/
#define MONEYPENNY_DEFAULT_EXPORT_FILE "mp_export.csv"



/* default webinterface for cups - normaly port 631 on localhost */
#define CUPS_CONFIG_PAGE "http://localhost:631/"


/** the numbers of the widgets on the widget stack
*/
typedef enum {
  WELCOME_WIDGET=1,
  HBCI_WIZARD,
  TOOLBOX_WIDGET,
  HARDWARE_WIDGET,  
  LOGIN_WIDGET,
  // BALANCES_WIDGET,
  ACCOUNT_INFORMATION_WIDGET,
  TRANSACTIONS_WIDGET,
  GET_TRANSACTIONS_WIDGET,
  TRANSACTIONS_PRINTER_WIDGET,
  INILETTER_WIDGET,
  SINGLE_TRANSFER_WIDGET,
  ACCEPT_TRANSACTION_WIDGET,
  DATED_TRANSFERS_WIDGET,
  DATED_SINGLE_TRANSFER_WIDGET,
  STANDING_ORDERS_WIDGET,
  START_STANDING_ORDER_WIDGET,
  HELP_WIDGET,
  KNOWNBANKS_WIDGET,
  NULL_WIDGET,
  CHANGE_ACCOUNT_NAME_WIDGET,
  TRANSFER_TEMPLATES_WIDGET,
  PRINTER_CONFIG_WIDGET,
  JOBS_WIDGET,
  LOG_WIDGET, 
  ADD_ACCOUNT_WIDGET,
  TRANSACTION_INFORMATION_WIDGET,
  EXPORT_WIDGET,
  SQL_QUERY_WIDGET,
  SQL_RESULT_WIDGET,
  BURN_WIDGET  		// delete after CeBIT
} WIDGET;

/**The HBCI communication modes
*/
typedef enum{
  HBCI_MODE_ERROR,
  HBCI_MODE_DDV,
  HBCI_MODE_RDH
}HBCI_MODE;

/**
* HoBIT uses ktoblzcheck to validate accounts 
* This means: If KtoBlzCheck does not know a 
* BankId, Transactions to those accounts will be
* skipped. During testing the default setting 
* musst be false, because KtoBlzCheck will tell
* hobit that the testbank does not exist
*/
#define SKIP_FALSE_TRANSACTIONS false



/**
* Money Penny comes as a linux version and on live cd
* The live cd version has special features which are not
* available in the linux version. set this param true
* to build a live-cd version.
*/
#ifdef LIVE_CD
  #define USE_LIVE_CD_FEATURES true
#else
  #define USE_LIVE_CD_FEATURES false
#endif

/**
* Just a little addon for the CEBIT-Version
*/
#define USE_CEBIT_FEATURES false


/** The error codes for hobit functions.
*/
 
typedef enum {
    HobitErrorSuccess=0,
    HobitErrorArguments,
    HobitErrorLoad,
    HobitErrorSave,
    HobitErrorInvalidArgs,
    HobitErrorInconsistentSystem,
    HobitErrorExecution,
    HobitErrorReadingfile,
    HobitErrorWritingfile,
    HobitErrorSequence,
    HobitErrorNotConnected
} HobitError;


/** The state values for the internet connection.
 *  Used by HoBIT_Connector.
 */
typedef enum{
   ConnectionUntested,
   ConnectionConnecting,
   ConnectionFailed,
   ConnectionConnected,
   ConnectionClosed,
   ConnectionWasOk
} ConnectionState; 

/** The type of storage medium for the config and data.
*/
typedef enum{
 STORAGE_UNKNOWN,
 STORAGE_FLOPPY,
 STORAGE_USB,
 STORAGE_ZIP,
 STORAGE_MANUALLY
}StorageType;


/** Exit Codes of the Application
*/
typedef enum{
 EXITCODE_SUCCESS,
 EXITCODE_WRONG_PASSWORD,
 EXITCODE_FAILURE,
 EXITCODE_RESTART
}ExitCode;








#endif

