/***************************************************************************
                          guidatedtransfers.cpp  -  description
                             -------------------
    begin                : Do Nov 11 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "guidatedtransfers.h"
#include "datedtransfer.h"
#include <qdatetime.h>
#include <qcombobox.h>
#include <qlistview.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include "listviewitems.h"

  /**
  *this is the constructor for the dialog
  *
  *@param main
  *@param parent the parent dialog of this widget
  *@param name widgets title
  *@param hobit
  */
GuiDatedTransfers::GuiDatedTransfers(QWidget *parent, GuiMain* guimain, const char *name, HoBIT* hobit)
: GuiDatedTransfersBase(parent,name) {
  _hobit = hobit;
  _main = guimain;
  _DatedTransferList.setAutoDelete(true);
  
//   setTabOrder(comboBoxAccounts, listViewTransactions);
   setTabOrder(tableDatedTransfers, buttonNewDatedTransfer);
   setTabOrder(buttonNewDatedTransfer, buttonDeleteDatedTransfer);
   setTabOrder(buttonDeleteDatedTransfer, buttonGet);   
}

GuiDatedTransfers::~GuiDatedTransfers(){
   qDebug("Destructor GuiDatedTransfers finished");
}



/** 
* updates the listview depending on the selected account 
*/
void GuiDatedTransfers::slotChangeAccount(){

    Date fromdate;
    Date todate;

    QDate currentDate;
    currentDate = currentDate.currentDate();

    QString accountId = _main->getCurrentAccount();

    _DatedTransferList.clear();
    setList(_hobit->listDatedTransfers(   0,
                                          "*",
                                          accountId.ascii(),
                                          ""));  
    update();
}


/** 
* create list for dated transfers
*/
bool GuiDatedTransfers::setList(list<Pointer<DatedTransfer> > datedtransfers){

  int row = 0;
  DatedTransferItem * item;
  
  tableDatedTransfers->hide();
  tableDatedTransfers->setNumRows(0);
  _DatedTransferList.clear();
  
  list<Pointer<DatedTransfer> >::iterator  dtIter;

  for (dtIter=datedtransfers.begin(); dtIter!=datedtransfers.end(); dtIter++){
    
    item = new DatedTransferItem(tableDatedTransfers, row, *dtIter);
    _DatedTransferList.append(item);
    row++;
  }

  

  tableDatedTransfers->adjustColumn(0);
  tableDatedTransfers->adjustColumn(2);
  tableDatedTransfers->setColumnStretchable(1, true);
  tableDatedTransfers->show();
//update();
return true;

}


/** 
* creates job of dated transfers and send 
*/
void GuiDatedTransfers::slotGet(){
      _hobit->createJobGetDatedTransfers();
      if(_main->askForSend())
       _main->slotSendNow();
}


 /** 
 * executed when user performs a double click on a list item
 * parameters for check which item is selected
 */
void GuiDatedTransfers::slotSelectItem(int row, int col, int button, const QPoint & mousePos){
  DatedTransferItem* dtr = _DatedTransferList.at(row);
  if (dtr){
   //Pointer<StandingOrder> dt = dtr->datedTransfer(); 
   //emit showInformation(dt);
 }
}


/** 
* raises widget of dated single transfer formular 
*/
void GuiDatedTransfers::slotNewDatedTransfer(){
emit raiseWidget(DATED_SINGLE_TRANSFER_WIDGET);
_main->refresh();
}


/** 
* deletes an existing dated transfer job
*/
void GuiDatedTransfers::slotDeleteDatedTransfer(){

int currentRow = tableDatedTransfers->currentRow();
DatedTransferItem* dti = _DatedTransferList.at(currentRow);
if (tableDatedTransfers->currentSelection() != -1 && dti){
  Pointer<DatedTransfer> dt = dti->datedTransfer();

   string purpose = dt.ref().purpose();

   QString text = "Sie sind in Begriff folgenden Auftrag zu stornieren: <br> <br>";
   text += QString(purpose.c_str());
   text += "<br> <br> Sind Sie sich sicher, dass sie das wollen?";
   

  bool  keep =  QMessageBox::question(  this,
                                                              "Auftrag stornieren?",
                                                              text,
                                                             "&Ja", "&Nein", QString::null, 0, 1 );

  if (!keep){
     _hobit->createJobDeleteDatedTransfer(dt);
     _hobit->createJobGetDatedTransfers();
     if(_main->askForSend())
      _main->slotSendNow();
  }
}
else QMessageBox::critical(this, tr("problem"), tr("You did not select an item"));
}



/*!
    \fn GuiDatedTransfers::refresh()
 */

/** 
* set accountlabel to current account
*/
void GuiDatedTransfers::refresh()
{
//    comboBoxAccounts->clear();
//    list<Pointer<Account> > accounts = _hobit->listAccounts();
//    list<Pointer<Account> >::iterator  accit;
/*
  for (accit=accounts.begin(); accit!=accounts.end(); accit++){

      string accountId = (*accit).ref().accountId();
      comboBoxAccounts->insertItem( QString(accountId.c_str()) );
      }

  if (accounts.empty())
          comboBoxAccounts->insertItem(QString("Keine Konten vorhanden"));
  */
  
  textLabelAccount->setText(_main->getCurrentAccount());
  if (_main->getCurrentAccount() != NULL)
		slotChangeAccount();
}


/** 
* Read property of QPtrList<TransactionItem> _TransactionList. 
*/
QPtrList<DatedTransferItem>  GuiDatedTransfers::getDatedTransferList()const{
	return _DatedTransferList;
}
