/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guifindmedium.h"

#include <stdlib.h>

#include <qpushbutton.h>
#include <qfile.h>
#include <qstringlist.h>
#include <qradiobutton.h>
#include <qregexp.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qdir.h>
#include <qcursor.h>
#include <qprocess.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qframe.h>
#include <qcursor.h>

GuiFindMedium::GuiFindMedium(QWidget * parent)
 : GuiFindMediumBase(parent)
{
 _Storage = STORAGE_UNKNOWN;
 buttonOk->setEnabled(false);
 connect(&_Timer, SIGNAL(timeout()), this, SLOT(slotTimerDone()));
 _TimeToShow = 2;
 _IsWritable = false;
 _MkDirStarted = false;
 _SkipTimer = false;
 frameMore->hide();
}

GuiFindMedium::~GuiFindMedium()
{
 qDebug("Destructor GuiFindMedium finished");
}

/** Read property of StorageType m_Storage. 
*/
const StorageType& GuiFindMedium::getStorage()const{
	return _Storage;
}

/** Write property of StorageType m_Storage. 
*/
void GuiFindMedium::setStorage( const StorageType& _newVal){
	_Storage = _newVal;
}

/** Searches for new partitions
*/
void GuiFindMedium::slotTimerDone(){  
   if(_SkipTimer)//do not interfere with running processes
     return; 
   if(_Storage == STORAGE_UNKNOWN){ // search for device
     if(radioButtonFloppy->isChecked())
        searchForFloppy();
     else if(radioButtonUsb->isChecked())
        searchForUsbDevice();    
   }//end if STORAGE_UNKNOWN
   else 
   { // device found or selected, wait for /proc/mounts to be updated
     if(_Storage == STORAGE_USB){
       
        if(USE_LIVE_CD_FEATURES)
           runProcess("sudo mount " +_DeviceName + " " + QString(MONEYPENNY_KNOPPIX_MOUNTPOINT));
        labelStatus->setText(tr("New USB device found. Checking for writability. This will take a few seconds.<br> Please wait."));
        setCursor(Qt::waitCursor);
        labelStatus->setPaletteBackgroundColor(yellow);
      
        _MountPoint = findMountPoint("/proc/mounts", _DeviceName); // this will not work immediately, 
     
                                                               // wait for the next timeout
       if(_MountPoint.length() > 0){ 
         qDebug("USB device found: %s", _MountPoint.latin1());
         labelStatus->setText(tr("USB device is writable. Please proceed."));
         setCursor(QCursor(Qt::ArrowCursor));
         labelStatus->setPaletteBackgroundColor(green);
         buttonOk->setEnabled(true);
       }
       else{          
       }
      }//end USB
      else if(_Storage == STORAGE_FLOPPY){ 
        _MountPoint = findMountPoint("/proc/mounts", _DeviceName);
        if(_MountPoint.length() > 0){ 
           qDebug("Floppy found: %s", _MountPoint.latin1());
           labelStatus->setText(tr("A floppy disc has been found. Please proceed."));
           setCursor(QCursor(Qt::ArrowCursor));
           labelStatus->setPaletteBackgroundColor(green);
           buttonOk->setEnabled(true);
        }//end length > 0
      }//end floppy  
      else if(_Storage == STORAGE_MANUALLY){ 
        _MountPoint = findMountPoint("/proc/mounts", _DeviceName);
        if(_MountPoint.length() > 0){ 
           qDebug("Storage device selected: %s", _MountPoint.latin1());
           labelStatus->setText(tr("A writable storage device has been found. Please proceed."));
           setCursor(QCursor(Qt::ArrowCursor));
           labelStatus->setPaletteBackgroundColor(green);
           buttonOk->setEnabled(true);
        }//end length > 0
      }//end storage manual  
      if((_MountPoint.length() > 0)&&! _IsWritable) {//device is not writable
       qDebug("Error: Device is write protected.");
       setCursor(QCursor(Qt::ArrowCursor));
       labelStatus->setText(tr("The storage device is write protected. Please remove it, make it writable and try again."));
       labelStatus->setPaletteBackgroundColor(red);
     }
   }//end else  device found
   
   if((_MountPoint.length() > 0) && _IsWritable) //everything is complete, mountpoint found for writable device
    _TimeToShow --; //countdown
   if(_TimeToShow <= 0) 
     accept();
}

/** Starts the timer loop
*/
void GuiFindMedium::showEvent(QShowEvent * se){
    _Timer.start(1000);
    radioButtonUsb->setChecked(true);
    labelStatus->setText(tr("Waiting for a USB device to be plugged in now."));
    labelStatus->setPaletteBackgroundColor(yellow);
    _TimeToShow = 2; 
    _Storage = STORAGE_UNKNOWN;
    _IsWritable = false;
    _MountPoint = "";
}

/** Stops the timer loop
*/
void GuiFindMedium::hideEvent(QHideEvent * se){
   _Timer.stop();
}

/** Tries to read from floppy or USB device
*/
void GuiFindMedium::slotSelectionChanged(){
 if(radioButtonFloppy->isChecked()){
   labelStatus->setText(tr("Waiting for a floppy disk to be inserted now."));
   labelStatus->setPaletteBackgroundColor(yellow);
 }
 else if(radioButtonUsb->isChecked()){
   labelStatus->setText(tr("Waiting for a USB device to be plugged in now."));
   labelStatus->setPaletteBackgroundColor(yellow);
 }
 _Storage = STORAGE_UNKNOWN;
 _MountPoint = "";
}

/** Searches for a mounted floppy
*/
bool GuiFindMedium::searchForFloppy(){
 if(USE_LIVE_CD_FEATURES){ //cd into the floppy dir, this will automatically mount the floppy on KNOPPIX
   runProcess("cd /mnt/floppy");
 }
 QString dirName = findMountPoint("/proc/mounts","/dev/fd0"); //first try the devicename
 if(dirName.isEmpty())
   dirName = findMountPoint("/proc/mounts","floppy"); //there may be other floppy drives, should be mounted on .../floppy/...
 
 if(!dirName.isEmpty()){//give up searching, look what we have found
    QDir dir(dirName); 
    QStringList files = dir.entryList();
    if(files.count() > 0){
      _Storage = STORAGE_FLOPPY; //do not set _MountPoint here, it is set in slotTimerDone
      return true;
    } 
    else return false;
 }   
 else return false;
}

/** Searches for a USB-device. Processes entries in /proc/partitions
*/
bool GuiFindMedium::searchForUsbDevice(){
 bool result = false;
 bool isNew = false;
 QStringList newList;
 QStringList sections;
 QRegExp rxDevice("[a-z]{3,3}[0-9]{1,1}"); //3 letters and one digits e.g. sda1
 QRegExp rxSplit("[\\s]"); //any whitespace
 QString newLine, oldLine; //for easy debugging
 QFile partitionsFile("/proc/partitions");
 if(partitionsFile.open(IO_ReadOnly)){
   QTextStream s(&partitionsFile);
   while(! s.atEnd()){ //read present partion list and save it in newList
     newLine = s.readLine();
     newList.push_back(newLine);
   }//end while
   partitionsFile.close();
 }//end if 
 //check for changes in list
 if(_PartitionList.count() > 0){
   if(_PartitionList.count() < newList.count()){
      for(uint nNewLine = 0; nNewLine < newList.count(); nNewLine ++){ // run through the whole list and check all entries
       isNew = true;
       newLine = newList[nNewLine];
       for(uint nOldLine = 0; nOldLine < _PartitionList.count(); nOldLine ++){//compare to all old lines
         oldLine = _PartitionList[nOldLine];
         if(newLine == oldLine) 
           isNew = false;
       }//end for nOLdLine
       if(isNew){  
         sections= QStringList::split(rxSplit, newLine);
         if(rxDevice.exactMatch(sections[sections.count() -1])){ //last column contains device name
           _DeviceName = "/dev/" + sections[sections.count() -1]; // this is only sda1 (uba1) at this point
           qDebug("Device name: %s", _DeviceName.latin1());
           _Storage = STORAGE_USB;
           
            if(USE_LIVE_CD_FEATURES){ // mount it yourself
              QString mountpoint(MONEYPENNY_KNOPPIX_MOUNTPOINT);              
              runProcess("sudo mkdir -m 777 " + mountpoint);    
            }//end CD
            
           result = true;
         }// end if match 
       }
     }//end for nNewLine
   }// end if <
   else if (_PartitionList.count() == newList.count()){
   //Nothing to do
 //    labelStatus->setText(tr("Waiting for new medium."));
 //    labelStatus->setPaletteBackgroundColor(yellow);
   }
   else if (_PartitionList.count() > newList.count()){
     labelStatus->setText(tr("A storage medium has been removed. Please insert your storage medium now."));
     labelStatus->setPaletteBackgroundColor(yellow);
     _MountPoint = "";
     _IsWritable = false;
  }//end if >
 }//end if > 0
 _PartitionList = newList; //save present list
 return result;
}

/** Searches the mountpoint in mtab or /proc/mounts and tries to write a file.
  * Sets _DeviceName 
  */
QString GuiFindMedium::findMountPoint(const QString & configFile, const QString & deviceName){
 QString result, line, mountflags, testFileName;
 QStringList entries;
 bool found = false;
 uint n =0;
 
 if(USE_LIVE_CD_FEATURES && (_Storage != STORAGE_USB))
  runProcess("sudo mount " + deviceName);
  
 QFile file(configFile);
  
 if(file.open(IO_ReadOnly)){
   QTextStream s(&file);
   while((! s.atEnd()) && (!found)){
     line = s.readLine();
     if(line.find(deviceName, 0, false)>=0){ //search for the name, whereever it is 
       qDebug("Device line: %s", line.latin1());
       entries= QStringList::split(" ", line);
       n = entries.count();
       result = entries[1];
       mountflags = entries[3];
       _IsWritable = mountflags.find("rw", 0) >= 0;
       testFileName = result + "/moneypenny.tst";
       qDebug("Testfile: %s", testFileName.latin1());
       QFile testFile(testFileName);
       _IsWritable = testFile.open(IO_WriteOnly); //do not rely on rw in /proc/mounts, try to write!
       if(_IsWritable)      //an empty file has been written
         testFile.remove(); //remove it
       _DeviceName = (*entries.at(0)); //this is the complete name e.g. /dev/sda1
      }    
   }//end while
   file.close();
 }//end if 
 return result;
}

/** Read property of QString _CryptDirName. 
*/
const QString& GuiFindMedium::getCryptDirName()const{
	return _CryptDirName;
}

/** Read property of QString _MountPoint. 
*/
const QString& GuiFindMedium::getMountPoint()const{
	return _MountPoint;
}

/**
*shows a short help message in a mesagebox.
*/
void GuiFindMedium::slotHelp(){
   QString msg =
   tr("MoneyPenny needs the removable storage medium in order to save data and key files.<br>\
   Finding a USB device is somewhat tricky, because there may be many USB-devices. \
   The easiest way is to look for a freshly plugged in device.<br>\
   Therefore it is important to switch on the computer <b>without</b> the USB device and plug it in, when MoneyPenny asks you to do so.\
   If you have already plugged in your USB device, remove it and plug it in again, after the removal has been detected.<br>\
   A floppy disk will be found, if it has been inserted, regardless of the insertion time. \
   <p>The program will proceed automatically a few seconds after a medium has been found.</p>");
   QMessageBox::information(this, "MoneyPenny Help", msg);
}

/** Read property of bool _IsWritable. 
*/
const bool& GuiFindMedium::isWritable()const{
	return _IsWritable;
}

/** Closes dialog and creates crypt-directory, if _IsWriteable is true. Else repeats search.
*/
void GuiFindMedium::accept(){
 if((_Storage != STORAGE_UNKNOWN) && _IsWritable){
  _CryptDirName = _MountPoint + "/" + QString(MONEYPENNY_MAIN_DIR);
   QDir dir(_CryptDirName);
  if(!dir.exists())
     dir.mkdir(_CryptDirName, true);
  if(!dir.exists()){
    QMessageBox::critical(this, tr("Critical Error"), tr("Could not create data directory: ") + _CryptDirName);  
    return;  
  }
  GuiFindMediumBase::accept();
 }
}

 /** Resets the flags and restarts timer
 */
 void GuiFindMedium::slotTryAgain(){
    _Storage = STORAGE_UNKNOWN;
    buttonOk->setEnabled(false);
    _IsWritable = false;
    _MkDirStarted = false;
    _SkipTimer = false;
    _DeviceName = "";
    _MountPoint = "";
    _CryptDirName = "";
    _PartitionList.clear();
 }
 
 /**Opens an Xterm and a file dialog
 */
 void GuiFindMedium::slotSearchManually(){
  _SkipTimer = true;
  QMessageBox::information(this, tr("How To Search"),
  tr("Sorry for not finding your medium. MoneyPenny will open a shell for experts next.\
   <br>Search the name of your usb device in /proc/partitions, mount it to /mnt/usb-moneypenny and exit the shell.\
   <br>Then a file dialog pops up. Select your mountpoint in this dialog and click Ok."));
  if(USE_LIVE_CD_FEATURES)
    system("sudo xterm");
  else 
    system("xterm");
  QString s = QFileDialog::getExistingDirectory(
                 "/mnt",
                 this,
                 "Select your mountpoint",
                 "Choose a directory",
                 TRUE );
  _Storage = STORAGE_MANUALLY;
  _MountPoint= s; //take the mountpoint for devicename 
  _IsWritable = true;
  _SkipTimer = false;
  accept();
 }
 
 /**Shows the extra buttons
 */
 void GuiFindMedium::slotMore(){
  setGeometry(x(), y(), width(), height() + 50);
  frameMore->show();
 }
 
/** Runs the specified process, spins the event loop and sets the _SkipTimer flag, so that timer events are not processed.
*/
void GuiFindMedium::runProcess(const QString & command){
   QCursor oldCursor = cursor();    
   _SkipTimer = true; //do not process pending timer events
   setCursor(WaitCursor);
   qDebug("Starting process: %s", command.latin1());
   system(command.latin1());
   _SkipTimer = false; 
   setCursor(oldCursor);
}
