/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guihbciwizard.h"
#include "guifindmedium.h"
#include "keyfileopenhbci.h"


#include "list"
#include "string"
#include "math.h"
using namespace std;

#include <qlineedit.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qradiobutton.h>
#include <qmessagebox.h>
#include <qcolor.h>
#include <qlabel.h>
#include <qstring.h>
#include <qregexp.h>
#include <qvalidator.h>
#include <qprogressbar.h>
#include <qcheckbox.h>
#include <qfile.h>
#include <qlistbox.h>
#include <qprocess.h>
#include <qinputdialog.h>

// The numbers for the wizard pages
typedef enum{
 WELCOME_PAGE,
 INTERNET_PAGE,
 BANK_PAGE,
 USER_PAGE,
 MEDIUM_PAGE,
 CHECK_PAGE,
 PASSWORD_PAGE,
 CONNECT_PAGE,
 FINGERPRINT_PAGE,
 ACCOUNTS_PAGE,
 SEND_KEY_PAGE,
 FINISH_PAGE
}PAGE;

/**
*this is the constructor. Reads bank list files and sets lists.  
*
*@param main 
*@param parent the parent dialog of this widget
*@param name widgets title
*@param hobit
*/
GuiHbciWizard::GuiHbciWizard( QWidget * parent, const char *name, HoBIT* hobit)
    :GuiHbciWizardBase(parent, name)
{
  _parent = parent;
  _hobit = hobit;
  _MediumDir = "";
  _MediumChanged = false;
  _created = false;
  _KeyFound = false;   
  _ChangingPassword = false; 
  setModal(false);
  showExtension(false);

  cancelButton()->hide();
  finishButton()->hide();
  buttonSkipDataInput->hide();
  setFinishEnabled(page( FINISH_PAGE ), true);
  lineEditPort->setValidator(new QIntValidator(0, 65535, lineEditPort));
  
    // button internationalisation
  cancelButton()->setText(tr("&Cancel"));
  finishButton()->setText(tr("&Finish"));
  nextButton()->setText(tr("&Next"));
  backButton()->setText(tr("&Back"));
  helpButton()->setText(tr("&Help"));

  _connectionTestSuccessful = false;
  _sendingKeysSuccessful = false;

  _hobit = hobit;

  _banks = _hobit->listOfKnownHbciBanks();//read bank list file
  list<HoBIT_Bank>::iterator bit;

  if (_banks.empty()) new QListViewItem(listViewBanks, tr("List with HBCI bankdata not found: ") + 
    MP_DIR()  + QString(MONEYPENNY_BANKDATA_DIR) + "/" + QString(MONEYPENNY_HBCI_BANKDATA_FILE), "", "");

  for (bit = _banks.begin(); bit != _banks.end(); bit++)
  {
    HoBIT_Bank & bank =(*bit);
    new QListViewItem(  listViewBanks, 
			QString::fromUtf8(bank.getBankId()), 
			QString::fromUtf8(bank.getBankName()), 
			QString::fromUtf8(bank.getLocation()), 
			QString::fromUtf8(bank.getServer()));
  }

  buttonSelectFile->hide();
  
  
  // timer for refreshing card reader list
  _timerFindCardReader = new QTimer(this);
  connect( _timerFindCardReader, SIGNAL(timeout()), SLOT(slotFindCardReaders()) );    
  
  widgetStack->raiseWidget( WELCOME_PAGE );  
}


/** destructor */
  GuiHbciWizard::~GuiHbciWizard(){
  qDebug("Destructor GuiHbciWizard finished");
}


/**
* when user typed in bankid, look what bank it is and show it with name and code in lineedit
*/
void GuiHbciWizard::slotBankSelected()
{
  QListViewItem* it = listViewBanks->currentItem();
  QString bankCode = it->text(0);
  QString bankName = it->text(1);
  QString bankLocation = it->text(2);
  QString bankServer = it->text(3);

  lineEditBankCode->setText(bankCode);
  lineEditBankName->setText(bankName + ", " + bankLocation);
  lineEditBankServer->setText(bankServer);
  slotUpdateBankDataPage();  
}


/**
* function for changing CustomerId 
*/
void GuiHbciWizard::slotChangeCustomer()
{
  if ( lineEditUserId->text().isEmpty())
    setNextEnabled(currentPage(), false);
  else setNextEnabled(currentPage(), true);
}

/** Copies text from UserId to CustId
*/
void GuiHbciWizard::slotCopyUserId(){
  lineEditCustId->setText(lineEditUserId->text());
}

/**
* function for check if typed-in bank is existing
*/
void GuiHbciWizard::slotSearchBankId()
{
  bool bankFound = false;
  list<HoBIT_Bank>::iterator bit;
  for (bit = _banks.begin(); bit != _banks.end(); bit++)
  {
    QString bankId = (*bit).getBankId();

    if (bankId == lineEditBankCode->text())
    {
      lineEditBankName->setText((*bit).getBankName()+ ", " + (*bit).getLocation());
      lineEditBankServer->setText((*bit).getServer());
      lineEditBankCode->setPaletteBackgroundColor(green);
      lineEditBankName->setPaletteBackgroundColor(green);
      lineEditBankServer->setPaletteBackgroundColor(green);
      bankFound = true;
    }
  }
  if(!bankFound)
  {
    if (lineEditBankCode->text().length() >= 8){
      QColor orange(255, 164, 0);
      lineEditBankName->setText(tr("Could not find a bank with the code you input."));
      lineEditBankServer->setText(tr("May be, that your bank has no HBCI Server."));
      lineEditBankCode->setPaletteBackgroundColor(orange);
      lineEditBankName->setPaletteBackgroundColor(orange);
      lineEditBankServer->setPaletteBackgroundColor(orange);
    }  
    else {
     lineEditBankName->setText("");
     lineEditBankServer->setText("");
     lineEditBankCode->setPaletteBackgroundColor(yellow);
     lineEditBankName->setPaletteBackgroundColor(yellow);
     lineEditBankServer->setPaletteBackgroundColor(yellow);
    } 
  }
  slotUpdateBankDataPage();  
}


/**
* function for slot new medium
*/
void GuiHbciWizard::slotNewMedium( int state)
{
  if(state != QButton::On)
   return;
  // stop searching for card readers
  _timerFindCardReader->stop();
  widgetStack->raiseWidget(0);
  buttonSelectFile->hide();
  _sendingKeysSuccessful = false;
  _created = false;
  getKeyFileName(); //allow to change medium type
  lineEditMediumPath->setPaletteBackgroundColor(green);
  if(lineEditMediumPath->text() == "")
  lineEditMediumPath->setText(_MediumDir+ "/" + lineEditUserId->text() + ".key");
  setNextEnabled(currentPage(), true); 
  textLabelHint->setText(tr("See below for your new key's name. You may edit the filename here. <br> Please proceed to the next page.")); 
}

/**
* function for slot existing medium
*/
void GuiHbciWizard::slotExistingMedium( int state)
{
  if(state != QButton::On)
   return;
  // stop searching for card readers
 _timerFindCardReader->stop();
  widgetStack->raiseWidget(0);
  buttonSelectFile->show();
  _sendingKeysSuccessful = true;
  labelSendKeys->setText(tr("You need not send the key again, if it has already been sent earlier."));
  getKeyFileName();
}

/**
* function for slot chip card  
*/
void GuiHbciWizard::slotChipCard( int state)
{
  if(state != QButton::On)
   return;
  // start the timer which searches card readers
   slotFindCardReaders();
 // _timerFindCardReader->start( 5000, FALSE ); !!!!
  widgetStack->raiseWidget(4);
}

/**
* function for change the medium type, called twice, once for every button 
*/
void GuiHbciWizard::slotChangeMediumType()
{
  if(isVisible()&& _MediumChanged)
   getKeyFileName();
  _MediumChanged = true;
}

/**
* function for choose keyfiles
*/
void GuiHbciWizard::slotSearchFile()
{
  QString fileName;
  QString keyFiles = tr("HBCI key files (*.key)");
  QString userId = lineEditUserId->text();
  fileName = QFileDialog::getOpenFileName(lineEditMediumPath->text(),keyFiles,this);
  if (!fileName.isEmpty()){
    QString password = QInputDialog::getText
       (tr("Import keyfile"), tr("Please input the password for your existing keyfile."), QLineEdit::Password);
    KeyFileOpenHbci keyFile(fileName, password);
    if(keyFile.readFile()){
        QFileInfo info (fileName);
	fileName = info.fileName(); //strip dir
	fileName = _MediumDir + "/" + fileName;
	qDebug(fileName.latin1());
	keyFile.writeFile(fileName);
	lineEditMediumPath->setPaletteBackgroundColor(green);
	lineEditMediumPath->setText(fileName);
	setNextEnabled(currentPage(), true);
    }
    else
     QMessageBox::critical(this, tr("Error"),tr("Could not read keyfile. Wrong password or unsupported file format."));
  }
     
}

/**
* function for a test connection to the bank, the user gets informations about the status 
*/
void GuiHbciWizard::slotTestConnection()
{
  // get information from the input mask
  string userid = lineEditUserId->text().ascii();
  string bankid = lineEditBankCode->text().ascii();
  string custid = lineEditCustId->text().ascii();
  string server = lineEditBankServer->text().ascii();
  string mediumPath;
  HobitError error;
  bool hasErrors = false;

  string mediumtype;
  int country = MONEYPENNY_DEFAULT_COUNTRY;
 // HoBIT_Connector connector(this, lineEditServer->text(), 3000 );


  if (radioButtonChipCard->isOn())
  {
    mediumtype = "DDVCard";
    mediumPath = "Security Card";
  }
  else if (radioButtonExistingMedium->isOn())
  {
    mediumtype = "RDHFile";
    mediumPath = lineEditMediumPath->text().ascii();
  }
  else if (radioButtonNewMedium->isOn())
  {
    mediumtype = "RDHFile";
    mediumPath = lineEditMediumPath->text().ascii();
  }
  else
  {
    QMessageBox::critical(this, tr("Data missing"), tr("A medium that contains the key must be specified."));
    return;
  }
  
  if (mediumtype == "DDVCard"){
  // import user from card
  error =_hobit->importUserDDV(false,
           	               country,
                               bankid,
                      	       custid,
                               custid,
                               server);
  if (error != 0)
  	  hasErrors = true;
  else{
	QString info = tr("Importing chipcard-data was successful. \n MoneyPenny will now try to establish a HBCI connection to your bank.\n\n");
 	_hobit->showLogText(info);
	QMessageBox::information(this, tr("Chipcard data could be read."), info);
 	}
   }
  
  else {
  // create user
  error = _hobit->createUser(  mediumPath,
          	               mediumtype,
                     	       false,
                     	       country,
                     	       bankid,
                     	       custid,
                     	       custid,
                     	       server);
  if (error != 0)
  	  hasErrors = true;
 }

if (!hasErrors){
  // create a job to get bank keys
  if (mediumtype != "DDVCard"){
  _hobit->showLogText(tr("Adding job: Get bank's public key"));
  error = _hobit->createJobGetKeys();
  if (error != 0)
    hasErrors = true;
   
  // now execute the job queue
  _hobit->showLogText(tr("Executing job queue."));
  error = _hobit->executeJobQueue();
  if (error != 0) hasErrors = true;


  if (hasErrors)
  {
    _hobit->showLogText(tr("Execution terminated with errors."));
    textLabelTestConnection->setText(tr("failed"));
    textLabelTestConnection->setBackgroundColor(Qt::red);
    _connectionTestSuccessful = false;
    setNextEnabled(page( CONNECT_PAGE ), false);
  }
  else
  {
    _hobit->showLogText(tr("Execution terminated successfully."));
    textLabelTestConnection->setText(tr("Connection test was successful."));
    textLabelTestConnection->setBackgroundColor(Qt::green);
    _connectionTestSuccessful = true;
    setNextEnabled(page( CONNECT_PAGE ), true);
    _hobit->showLogText((""));
    _hobit->showLogText(("----------------------------------------------------"));
    _hobit->showLogText((""));

    // create a job to get systemid
    _hobit->showLogText(tr("Adding job: System ID Renewal."));
    error = _hobit->createJobGetSystemId();
    if (error != 0) hasErrors = true;
  
    // now execute the job queue
    _hobit->showLogText(tr("Executing job queue."));
    error = _hobit->executeJobQueue();
    string hash = _hobit->getBankFingerPrint(MONEYPENNY_DEFAULT_COUNTRY, bankid, custid);
    cerr << "Bank Hash: " <<  hash;
    cout << "Bank Hash: " <<  hash;

    if (error != 0)
    {
    _hobit->showLogText(tr("Could not synchronize with the bank server."));
    _hobit->showLogText(tr("Most probably your public key was not yet sent or not activated by the bank."));
    }
  textLabelBankHash->setText(QString(hash.c_str()));
  }
}
// if in card mode
if (mediumtype == "DDVCard"){
  _hobit->showLogText(tr("Adding job: Get Accounts"));
  error = _hobit->createJobGetAccounts();
  if (error != 0)
    hasErrors = true;
   
  // now execute the job queue
  _hobit->showLogText(tr("Executing job queue."));
  error = _hobit->executeJobQueue();
  if (error != 0) hasErrors = true;

  if (hasErrors)
  {
    _hobit->showLogText(tr("Execution terminated with errors."));
    textLabelTestConnection->setText(tr("failed"));
    textLabelTestConnection->setBackgroundColor(Qt::red);
    _connectionTestSuccessful = false;
    setNextEnabled(page( CONNECT_PAGE ), false);
  }
  else
  {
    _hobit->showLogText(tr("Execution terminated successfully."));
    textLabelTestConnection->setText(tr("Connection test was successful."));
    textLabelTestConnection->setBackgroundColor(Qt::green);
    _connectionTestSuccessful = true;
    setNextEnabled(page( CONNECT_PAGE ), true);
  }
}
}
emit renewHoBIT();
update();
}

/**
* Dialog for send the keys. User gets informations about the status of sending
*/
void GuiHbciWizard::slotSendKeys()
{
  HobitError error;
  bool hasErrors = false;

  // create a job to get systemid
  _hobit->showLogText(tr("Creating job: Send your public key"));
  error = _hobit->createJobSendKeys();
  hasErrors = (error != HobitErrorSuccess);

  // now execute the job queue
  _hobit->showLogText(tr("Executing job queue."));
  error = _hobit->executeJobQueue();
  hasErrors = (error != HobitErrorSuccess);
    
  if (hasErrors)
  {
    _hobit->showLogText(tr("Execution terminated with errors."));
    labelSendKeys->setText(tr("Sending public key failed."));
    labelSendKeys->setBackgroundColor(Qt::red);
    _sendingKeysSuccessful = false;
  }
  else{
    _hobit->showLogText(tr("Execution terminated successfully."));
    labelSendKeys->setText(tr("Sending key terminated successfully."));
    labelSendKeys->setBackgroundColor(Qt::green);
    _sendingKeysSuccessful = true;
  }

  pageChanged();
 // _parent->refresh();
}

/** Fingerprints have been checked
*/
void GuiHbciWizard::slotFingerprintChecked(){
   if(checkBoxFingerprintOk->isChecked())
     checkBoxFingerprintOk->setPaletteBackgroundColor(green);
   setNextEnabled(page( FINGERPRINT_PAGE ), checkBoxFingerprintOk->isChecked());
}

/**
* Executed when the page changes 
*/
void GuiHbciWizard::pageChanged()
{
  //  Welcome page
  if (currentPage() == page( WELCOME_PAGE )){
    setNextEnabled(currentPage(), true);
    _MediumChanged = false;
    _created = false;
    _KeyFound = false;   
    _ChangingPassword = false;
    cancelButton()->show();
  }
  
  //  internet setup page
  if (currentPage() == page( INTERNET_PAGE )){
    buttonTestInternetSetup->setFocus();
    setNextEnabled(currentPage(), true);
    cancelButton()->hide();
  }
  
  //  bank data page
  if (currentPage() == page( BANK_PAGE )){
    lineEditBankCode->setFocus();
    slotUpdateBankDataPage();
    setNextEnabled(currentPage(), true);
    cancelButton()->hide();
  }
  
  //  customer data page
  if (currentPage() == page( USER_PAGE ))
  {
    lineEditUserId->setFocus();
    slotChangeCustomer();
    cancelButton()->hide();    
  }

  // medium data page
  if (currentPage() == page( MEDIUM_PAGE ))
  {
    if ( !radioButtonNewMedium->isChecked() &&
         !radioButtonExistingMedium->isChecked() &&
         !radioButtonChipCard->isChecked())
      setNextEnabled(currentPage(), false);  
    else {
    getKeyFileName();
    cancelButton()->hide();
    setNextEnabled(currentPage(), _KeyFound || radioButtonNewMedium->isChecked()); 
  }
}
  // show data information page
  if (currentPage() == page( CHECK_PAGE ))
  {
    textLabelInformation->setText(getDataString());
    if(radioButtonNewMedium->isOn()){
      setAppropriate(page(PASSWORD_PAGE), true); //skip the password and key generation page if no new medium is needed
      setAppropriate(page(ACCOUNTS_PAGE), false);
      }
    else {
       setAppropriate(page(PASSWORD_PAGE), false);
       setAppropriate(page(ACCOUNTS_PAGE), true);
    }
    cancelButton()->hide();
   }
  
  //password and key generation page
  if (currentPage() == page( PASSWORD_PAGE )){ 
     if(_ChangingPassword){
       textLabelOldPassword->show();
       lineEditOldPassword->show();
       lineEditOldPassword->setFocus();
       setNextEnabled(currentPage() , false);
       cancelButton()->show();
     }
     else{
       textLabelOldPassword->hide();
       lineEditOldPassword->hide();
       lineEditPassword1->setFocus();
       setNextEnabled(currentPage() , _created);
       cancelButton()->hide();
     }
  }
  
  //connection test page
  if (currentPage() == page( CONNECT_PAGE )){ 
    if (radioButtonChipCard->isChecked()){
      setAppropriate(page(PASSWORD_PAGE), false);
      setAppropriate(page(FINGERPRINT_PAGE), false);
      setAppropriate(page(SEND_KEY_PAGE), false);
      setAppropriate(page(FINISH_PAGE), true);
     _timerFindCardReader->stop();
     }
     else{
     setAppropriate(page(PASSWORD_PAGE), true);
     setAppropriate(page(FINGERPRINT_PAGE), true);
     setAppropriate(page(SEND_KEY_PAGE), true);
     }

    if (_connectionTestSuccessful) {
      setNextEnabled(currentPage(), true);
    }
    else setNextEnabled(currentPage(), false);
    cancelButton()->hide();
  }
  // fingerprint page
  if (currentPage() == page( FINGERPRINT_PAGE )){ 
      setNextEnabled(currentPage(), checkBoxFingerprintOk->isChecked());
      cancelButton()->hide();
 }
  //send key page
  if (currentPage() == page( SEND_KEY_PAGE )){ 
    if (_sendingKeysSuccessful){  
      setNextEnabled(currentPage(), true);
      labelSendKeys->setPaletteBackgroundColor(green);
    }  
    else{ 
      setNextEnabled(currentPage(), false);
      labelSendKeys->setPaletteBackgroundColor(yellow);
    }
   cancelButton()->hide();
 }

 //last page
  if (currentPage() == page( FINISH_PAGE ))
  {
    setFinishEnabled(currentPage(), true);
    finishButton()->setText(tr("&Finish HBCI Setup"));
    cancelButton()->hide();
  }
}


/**
* Executed when the user presses cancel button
*/
void GuiHbciWizard::reject()
{
  QMessageBox::question( _parent,
                         tr("Confirmation"),
                         tr("Cancel setup?<br> HBCI setup is not yet complete. Homebanking may not work. \
                             <br>Do you really want to cancel?"),
                         tr("&Yes"), tr("&No"),QString::null, 0, 1 );
}

/**
* Executed when the user presses the accept button
*/
void GuiHbciWizard::accept()
{
  _hobit->saveEnvironment();
  GuiHbciWizardBase::accept();
  
  if (radioButtonChipCard->isOn())
  emit setupSuccessful(HBCI_MODE_DDV);
  else
  emit setupSuccessful(HBCI_MODE_RDH);
}

/**
* Call help widget passing the appropriate help file.
*/
void GuiHbciWizard::help()
{
  switch(indexOf(currentPage()))
  { 
    case WELCOME_PAGE  : emit helpRequest("hbci_welcome.html"); break;
    case INTERNET_PAGE : emit helpRequest("hbci_internet.html"); break;
    case BANK_PAGE     : emit helpRequest("hbci_bank.html"); break;
    case USER_PAGE     : emit helpRequest("hbci_user.html"); break;
    case MEDIUM_PAGE   : emit helpRequest("hbci_medium.html"); break;
    case CHECK_PAGE    : emit helpRequest("hbci_check.html"); break;
    case PASSWORD_PAGE : emit helpRequest("hbci_password.html"); break;
    case CONNECT_PAGE  : emit helpRequest("hbci_connect.html"); break;
    case FINGERPRINT_PAGE: emit helpRequest("hbci_fingerprint.html"); break;
    case SEND_KEY_PAGE : emit helpRequest("hbci_send_key.html"); break;
    case FINISH_PAGE   : emit helpRequest("hbci_finish.html"); break;
    default            : emit helpRequest("hbci_welcome.html"); break;
  }
}

/** Show the help page on password security
*/
void GuiHbciWizard::slotHelpPassword(){
 emit helpRequest("security.html");
}

/** Sets password bar according to log of number of possible passwords.
*/
void GuiHbciWizard::slotPasswordStrength(){
 QString password = lineEditPassword1->text();
 double nChars = 0.0; // use double to prevent numerical overflow
 int pos;
 QRegExp rx0( "[0-9]{1,20}");
 QRegExp rx1( "[a-z]{1,20}");
 QRegExp rx2( "[a-z, A-Z]{1,20}" );
 QRegExp rx3( "[a-z, A-Z, 0-9]{1,20}");
 QRegExpValidator v0( rx0, NULL );
 QRegExpValidator v1( rx1, NULL );
 QRegExpValidator v2( rx2, NULL );
 QRegExpValidator v3( rx3, NULL );
 if(v0.validate(password, pos) == QValidator::Acceptable)
   nChars = 10;
 else if(v1.validate(password, pos) == QValidator::Acceptable)
   nChars = 26;
 else if(v2.validate(password, pos) == QValidator::Acceptable)
   nChars = 52;
 else if(v3.validate(password, pos) == QValidator::Acceptable)
   nChars = 62;
 else if(password.length() > 0) 
   nChars = 128;
 
 double nPasswords = exp( log(nChars) *double(password.length()));
 int progress = int(log(nPasswords) / log(10.0) * 70.0 / 16.0); //70% for 10E16
 if(password.length() == 0)
   progress =0;
 progressBarStrength->setProgress(progress, 100); 
 progressBarStrength->setPaletteBackgroundColor(green);
 if(progress < 70)
  progressBarStrength->setPaletteBackgroundColor(yellow);
 if(progress < 40)
  progressBarStrength->setPaletteBackgroundColor(red);
}


/** Checks password and creates a new key if ok
*/
void GuiHbciWizard::slotCheckPassword(){
 setNextEnabled(currentPage(), false);
 if(lineEditPassword1->text() == lineEditPassword2->text() ){
   if(progressBarStrength->progress() > 40){ 
       if(_ChangingPassword)
         setNextEnabled(currentPage(), changeKeyfilePassword());
       else if (radioButtonNewMedium->isOn() && !_created){   
         createNewKeyfile(); 
       }//end if !_created
     }//end if progress > 40
     else  QMessageBox::critical(this, tr("Error"), tr("The password is too weak, please use a longer one."));         
 }   
 else QMessageBox::critical(this, tr("Input Error"), 
                     tr("The passwords in the two edits are not identical. <br> Please input again."));  
}

/** Creates a html formatted QString containing the data input.
*/
QString GuiHbciWizard::getDataString(){
    // get information from the input mask
    QString userid = QString(lineEditUserId->text());
    QString bankid = QString(lineEditBankCode->text());
    QString bankname = QString(lineEditBankName->text());
    QString custid = QString(lineEditCustId->text());
    QString server = QString(lineEditBankServer->text());
    QString mediumPath;

    QString mediumtype;

    if (radioButtonChipCard->isChecked())
    {
      mediumtype = tr("DDV-Chip Card");
      mediumPath = QString(lineEditMediumChipPath->text());
    }
    else if (radioButtonExistingMedium->isOn())
    {
      mediumtype = tr("RDH file on ");
      if (radioButtonFloppy->isChecked())
        mediumtype += tr("Floppy Disc");
      else mediumtype += tr("Usb-Memory-Stick");
      mediumPath = QString(lineEditMediumPath->text());
    }
    else if (radioButtonNewMedium->isChecked())
    {
      mediumtype = tr("RDH file on ");
      if (radioButtonFloppy->isChecked())
        mediumtype += tr("Floppy Disc");
      else mediumtype += tr("Usb-Memory-Stick");
      mediumPath = QString(lineEditMediumPath->text());          
    }
    
    QString data;
    data =  tr("Customer ID: "); data += custid; data += "<br>";
    data += tr("User Id: "); data += userid; data += "<br> <br>";
    data += tr("At bank: "); data += bankname; data += "<br>";
    data += tr("Bank ID: "); data += bankid; data += "<br>";
    data += tr("Server address: "); data += server; data += "<br><br>";
    data += tr("Medium used for keys: "); data += mediumtype; data += "<br>";
    data += tr("File path to medium: "); data += mediumPath; data += "<br>";
    
    return data;
}

/** Updates the state of the controls on bank data page.
*/
void GuiHbciWizard::slotUpdateBankDataPage(){
    if ( lineEditBankCode->text().isEmpty() ||
         lineEditBankName->text().isEmpty() ||
         lineEditBankServer->text().isEmpty())
    {
      setNextEnabled(currentPage(), false);
    }
    else setNextEnabled(currentPage(), true);
}

/** Sets the corresponding radiobutton.(Floppy or Usb)
*/
void GuiHbciWizard::setStorageType(StorageType storage){
 switch(storage){
   case STORAGE_USB: radioButtonUsb->setChecked(true);break; //calls slotChangeMediumtype via connect
   case STORAGE_FLOPPY: radioButtonFloppy->setChecked(true); break; //calls slotChangeMediumtype
   default: radioButtonFloppy->setChecked(true); break; //calls slotChangeMediumtype
 }
 _MediumChanged = false; //reset flag. it has been set by slotChangeMediumtype
}

/** Write property for _MediumDir
*/
void GuiHbciWizard::setMediumDir(const QString & dir){
  _MediumDir = dir;
  lineEditMediumPath->setText(_MediumDir);
}

/** Searches for an existing key file or asks the user for a name.
*/
void GuiHbciWizard::getKeyFileName(){
    QString keyname;
    QString fileName;
    _KeyFound = false;
    
    if(_MediumChanged){
      GuiFindMedium guifindmedium(this);
       if(guifindmedium.exec() == QDialog::Accepted){
        _MediumDir = guifindmedium.getCryptDirName();
      }  
    }  
    if((lineEditMediumPath->text() == "") || radioButtonNewMedium->isChecked()){
      keyname = _MediumDir+ "/" + lineEditUserId->text().right(4) + ".key";
      lineEditMediumPath->setText(keyname);
    }  
    else 
      keyname = lineEditMediumPath->text();
    
    if(radioButtonExistingMedium->isChecked()){ //search for existing key
       QFile keyFile(keyname);
       if(keyname.endsWith(".key") && keyFile.exists()) //key file with default name found 
         _KeyFound= true;
       else{//search for any key file
         QFileInfo fileInfo(keyFile);
         if(! fileInfo.isDir())
           keyname = fileInfo.dirPath();
         QDir dir(keyname);
         QStringList fileList = dir.entryList();
         for(uint nFile = 0; nFile < fileList.count(); nFile++){
          fileName = fileList[nFile];
          if(fileName.find(".key") > 0){
            keyname = _MediumDir + "/" + fileName;
            _KeyFound = true;
            break;
          }//end if
         }//end for   
       }//end search for key      
       
       if(_KeyFound){
            setNextEnabled(currentPage(), true);
            lineEditMediumPath->setPaletteBackgroundColor(green);
            textLabelHint->setText(tr("<br><br>Your key has been found. Please go to the next page. "));
       }
       else{     
        QMessageBox::information(this, tr("Information"), tr("Could not find a *.key file in moneypenny_data directory. Please search manually."));
        textLabelHint->setText(tr(
"Hint:<br>\
MoneyPenny is running on Linux. There are no drive letters.\
/ is used instead of \\ in the path.  Please try the following paths:<br>\
For floppy disk medium try /mnt/floppy or /media/floppy, for USB stick medium try /mnt/sda1 or /mnt/usb or /media/sda1 or/media/usb"));
        keyname = _MediumDir;
       } 
       lineEditMediumPath->setText(keyname);
    }//search for key   
}

/** Checks the Internet setup. Code is copied from guiConnections
*/
void GuiHbciWizard::slotTestInternetSetup(){
 ConnectionState state;
 emit testInternet(lineEditUrl->text(), lineEditPort->text().toInt(), &state); //connected to guiMain::slotTestInternet
 switch (state){
  case ConnectionWasOk:{
         labelInternetState->setText(tr("Status: Internet setup ok."));
         labelInternetState->setPaletteBackgroundColor(green);
         break;
        }
  case ConnectionFailed:{
        labelInternetState->setText(tr("Status: Internet setup is not ok. Cables connected?"));
        labelInternetState->setPaletteBackgroundColor(red);
        break;
       }
  default:{ 
       labelInternetState->setText(tr("Status: Internet setup not yet tested."));
       labelInternetState->setPaletteBackgroundColor(yellow);
       break;
     }
 }
}

/** Calls Internet setup program.
*/
void GuiHbciWizard::slotInternetSetup(){
 emit setupInternet( HARDWARE_WIDGET ); //connected to guiMain::slotRaiseWidget
}


 /** show log widget
  */
  void GuiHbciWizard::slotShowDetails(){
    emit showLog(LOG_WIDGET);
  }

  
  
  
/** reads the user's previous configuration from the OpenHbci database */  
void GuiHbciWizard::evaluateStoredData(){
  
  Pointer<Bank> bank = _hobit->getBank();
  Pointer<User> user = _hobit->getUser();
  Pointer<Customer> customer = _hobit->getCustomer(); 
  Pointer<Medium> medium = _hobit->getMedium();
  
  QString mediumName;
  if(medium.isValid()){
    mediumName = medium.ref().mediumName().c_str();
    if(medium.ref().securityMode() == HBCI_SECURITY_RDH)
      radioButtonExistingMedium->setChecked(true);
    else
      radioButtonChipCard->setChecked(true);  
  }
  else //no valid medium available
    radioButtonNewMedium->setChecked(true);
   
  buttonSkipDataInput->hide();
  
  if (bank.isValid()){
    lineEditBankCode->setText(bank.ref().bankCode().c_str());
    lineEditBankName->setText(bank.ref().name().c_str());
    lineEditBankServer->setText(bank.ref().address().c_str());
    
    if (user.isValid()){
      lineEditUserId->setText(user.ref().userId().c_str());
      lineEditMediumPath->setText(mediumName);      
        
      if (customer.isValid()){
        lineEditCustId->setText(customer.ref().custId().c_str());  
        if(_hobit->listAccounts().empty()) //show the account error message and the skip button
          textLabelWelcome->setText(tr(
//Do not indent this multiline string          
"<font size=\"+1\"><h2>Welcome to MoneyPenny. </h2>\
Your configuration data have been found and read.\
However, there are no accounts in your configuration data.\
If you use a keyfile security medium (RDH,)then\
most probably your key was not yet activated by the bank,\
when you used MoneyPenny for the last time.<br>\
To obtain the account list from your bank,\
send the Ini-Letter to your bank and ask them to activate your key.\
Then run this wizard again.<br>\
Most banks send the account list during the connection test.\
However, some banks do not send account data.\
In this case, you will have to fill in the account form of this wizard yourself.\
</font>" ));
          buttonSkipDataInput->show();
          buttonSkipDataInput->setFocus();
        }//end if customer is valid
        else textLabelWelcome->setText(tr(
//Do not indent this multiline string          
"<font size=\"+1\"><h2>Welcome to MoneyPenny. </h2>\
 Your configuration data have been found and read.\
 However, there seems to be a problem with the customer data.\
 We suggest, that you run this wizard again.\
 </font>"
                                         ));
    }//end if user is valid
  }//end if bank is valid
}


/** Reads /etc/chipcard.conf and lists all card readers found*/
void GuiHbciWizard::slotFindCardReaders(){

QString path  = getenv("HOME");
path += "/";
path += MONEYPENNY_MAIN_DIR;
path += "/";
uint nLine = 0;

if(system("chipcard-tool list > " + path + "cardreaders.lst") !=0)
 QMessageBox::critical(this, tr("Error"), tr("Your libchipcard installation is not ok. Executing chipcard-tool failed."));

if (QFile::exists(path + "cardreaders.lst")){  
   labelReaders->clear();
   QString txt = "";
   
   QFile file(path + "cardreaders.lst");
       if ( file.open( IO_ReadOnly ) ) {
           QTextStream stream( &file );
           QString line;
           while ( !stream.atEnd() ) {
               nLine++;
               txt += stream.readLine() += "\n"; // line of text excluding '\n'
           }
           labelReaders->setText(txt);
        }  
        file.close();
   }
  if(nLine != 0){
    _sendingKeysSuccessful = true;
    setNextEnabled(currentPage(), true); 
    labelStatus->setText(tr("Using a DDV-Chipcard from your Bank"));
    labelSendKeys->setText(tr("Key is already known. Sending is not neccessary."));
  }
  else{
    labelReaders->setText(tr("No cardreader was found. Did you install libchipcard and the cardreader's driver?")); 
    setNextEnabled(currentPage(), false);
  }  
}

/** Opens card reader configuration tool */
void GuiHbciWizard::configureCardReader(){
  QProcess process(this);
  process.addArgument("kdesu");
  process.addArgument("kcardsetup");
  process.start();
}


  /** set up account list */
  bool GuiHbciWizard::findAccounts(){
	
   bool found = false;
   listBoxAccounts->clear();

    list<Pointer<Account> > accounts = _hobit->listAccounts();
    list<Pointer<Account> >::iterator  accit;

  if (accounts.empty()){
          listBoxAccounts->insertItem(tr("No accounts found"));
	 }
  else{ 
  	for (accit=accounts.begin(); accit!=accounts.end(); accit++){
             string accountId = (*accit).ref().accountId();
             listBoxAccounts->insertItem( QString(accountId.c_str()) );
        }
        found = true;
      }
 
 return found;
  }


/**
  *raises the add account widget 
  */
  void GuiHbciWizard::slotAddAccount(){
   emit addAccount();	
  }

  
void GuiHbciWizard::updateAccountList(){
  if (findAccounts()){
      textLabelAccounts->setText(tr("MoneyPenny has retrieved a list of your accounts from the bank server: "));
      pushButtonAddAccount->hide();
  }
}


/** Skip the data input pages, because data have already been read.
*/
void GuiHbciWizard::slotSkipDataInput(){
   showPage(page(MEDIUM_PAGE));
}

/** Changes the password for the hbci communication. Only valid for RDH files.
*/
void GuiHbciWizard::changePassword(){
  HBCI_MODE mode = _hobit->getHbciMode();
  switch (mode){
    case HBCI_MODE_DDV:{
               QMessageBox::information(this, tr("Sorry"), tr("Can not change the PIN of your chipcard." ));
               break;
             }
    case HBCI_MODE_RDH:{
               _ChangingPassword = true;
               setAppropriate(page(PASSWORD_PAGE), true);
               setAppropriate(page(CHECK_PAGE), false);
               showPage(page(PASSWORD_PAGE));
               pageChanged();
               break;        
             }
    default: {
               QMessageBox::critical(this, tr("Error"), tr("No valid security mode has been set. Please run HBCI-setup first." ));
               break;
             }
          
  }
}

/** Create a new keyfile.
*/
void GuiHbciWizard::createNewKeyfile(){
  //check for file errors
  QMessageBox::information(this, tr("Information"), tr("A new security medium will be created now"));
  QString fileName = lineEditMediumPath->text();
  QFileInfo fileInfo(fileName);
  QString dirName = fileInfo.dirPath();
  QFileInfo dirInfo(dirName);
  if(!dirInfo.exists()){
   QMessageBox::critical(this, tr("Error"), tr("Directory: ") + dirName +
   tr(" Does not exist.<br>Please go to page before, select an existing directory and try again."));
   return;
  }
  if(fileInfo.isDir())
    fileName += dirName + "/" + lineEditUserId->text() + ".key";
  fileInfo.setFile(fileName);  
  if(fileInfo.exists()){
   if(QMessageBox::warning(this, tr("Warning"), tr("Key file exists.<br>\
      If you overwrite your key, you must send a new ini-letter to your bank, in order to activate the new key.<br>\
      <b>Homebanking with the new key does not work before the bank has activated it.</b>"),
      tr("Keep existing key and old password"), // button0 text
      tr("Overwrite existing key")) //button1 text
     == 0){ // keep button was clicked
         _created = true;
         setNextEnabled(currentPage(), true);
         QMessageBox::information(this, tr("Information"), 
         tr("Using existing key file. Please proceed to next page.")); 
         return;
     }
     else if(!fileInfo.isWritable()){
      QMessageBox::critical(this, tr("Error"), tr("No write permission to file: ") + lineEditMediumPath->text() +
      tr("<br>Please make medium writable and try again."));
      return;
     }
  }//end if exists
  else if(!dirInfo.isWritable()){
    QMessageBox::critical(this, tr("Error"), tr("No write permission to directory: ") + dirName +
    tr("<br>Please make medium writable and try again."));
    return;
  }
 
  string userid = lineEditUserId->text().ascii();
  string custid = lineEditCustId->text().ascii();
  string bankid = lineEditBankCode->text().ascii();
  string mediumtype = "RDHFile";
  string mediumpath = lineEditMediumPath->text().ascii();
  string server = lineEditBankServer->text().ascii();
  int country = MONEYPENNY_DEFAULT_COUNTRY; //to be replaced for international use
   
  HobitError err = _hobit->createNewMedium( 
     mediumtype, false, country, bankid, userid, mediumpath, lineEditPassword1->text());
 
 // create user so that data will be found on reboot
 _hobit->createUser(mediumpath,
                    mediumtype,
                    false,
                    country,
                    bankid,
                    custid,
                    custid,
                    server);
  
  if (err == 0) {
  
     
     _hobit->saveEnvironment();
  
    _created = true;
    setNextEnabled(currentPage(), true); 
    QMessageBox::information(this, tr("Information"), tr("The security file has been created successfully"));       
    //emit restartNow();
     emit renewHoBIT();
    }
  else QMessageBox::critical(this, tr("Error"), tr("The security file could not be created"));         
}

/** Changes password of current keyfile.
*/
bool GuiHbciWizard::changeKeyfilePassword(){
  if(QMessageBox::question(this, tr("Confirmation"), tr("You are about to change the password of the current key file.") +
                             tr("\nDo you really want to change the password?"), 
                             QMessageBox::No| QMessageBox::Escape , QMessageBox::Yes, QMessageBox::NoButton )
                             == QMessageBox::Yes) 
  {
     QString oldPassword = lineEditOldPassword->text();
     QString newPassword = lineEditPassword1->text();
     QString filename = lineEditMediumPath->text();
     KeyFileOpenHbci keyfile(filename, oldPassword);
     if(!keyfile.readFile()){
       QMessageBox::critical(this, tr("Error"), tr("Could not read keyfile: ") + filename + tr("\n Wrong password or wrong file format?"));
       return false;
     }
     else{
       if(keyfile.changePassword(newPassword)){
         QMessageBox::information(this, tr("Information"),tr("Your HBCI password has been changed."));
         emit(renewHoBIT()); //make shure, that any old key data are deleted
         return true;
       }
       else{
         QMessageBox::critical(this, tr("Error"), tr("Could not change password. No write access to keyfile?"));
         return false;
       }// end final error
     }// end if OK 
  }//end confirmed
  else return false;
}


