/***************************************************************************
 *   Copyright (C) 2004 by Marcel Mulch                                *
 *   marcel@mulch-online.de                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guijobs.h"
#include <openhbci2/pointer.h>
#include <aqmoney2/accountjobs.h>
#include <aqmoney2/adminjobs.h>
#include <list>
#include <qlistview.h>
#include <qstring.h>
#include <gwenhywfar/db.h>


/**
*constructor of GuiJobs
*
*@param main a pointer on moneypenny´s main class
*@param parent a pointer on the main widgets widget stack
*@param name widgets title
*@param hobit the hobit object, needed for hbci functions
*/
GuiJobs::GuiJobs(QWidget *parent, GuiMain* guimain, const char *name, HoBIT* hobit)
    :GuiJobsBase(parent, name)
{
   _main = guimain;
   _hobit = hobit;
   listViewAccountJobs->setSorting(-1);
   listViewAdminJobs->setSorting(-1);
   refresh();
   
   _adminSelected = false;
   _accountSelected = false;
}


 /** destructor */
 GuiJobs::~GuiJobs(){
 qDebug("Destructor GuiJobs finished");
 }


/** 
*refreshes the job queues 
*/
void GuiJobs::refresh(){
    list<Pointer<AccountJob> > accountjoblist = _hobit->getAccountJobQueue();
    list<Pointer<AdminJob> > adminjoblist = _hobit->getAdminJobQueue();
    listViewAccountJobs->clear();
    listViewAdminJobs->clear();
    
    list<Pointer<AccountJob> >::iterator accountjobit;
    list<Pointer<AdminJob> >::iterator adminjobit;
    
    _count = 1;
    for (accountjobit = accountjoblist.begin(); accountjobit != accountjoblist.end(); accountjobit++){
    	analyzeAccountJob(*accountjobit);
    }
    
    _count = 1;
     for (adminjobit = adminjoblist.begin(); adminjobit != adminjoblist.end(); adminjobit++){
     analyzeAdminJob(*adminjobit);
    }

    // sort table
    listViewAccountJobs->setSorting(0);
    listViewAdminJobs->setSorting(0);
    listViewAccountJobs->sort();
    listViewAdminJobs->sort();
    listViewAccountJobs->setSorting(-1);
    listViewAdminJobs->setSorting(-1);
    
}

/**
*analyzing of the account job
*@param job pointer on AccountJob
*/
void GuiJobs::analyzeAccountJob(Pointer<AccountJob> job)
{
   QString name = QString(job.ref().name().c_str());    
   QString desc = name;
         
   if (name == "JOBGetBalance"){
      desc = tr("Get all balances for account ");
      desc += QString(job.ref().accountId ().c_str());
   }
   else if (name == "JOBGetTransactions"){
      desc = tr("Get all Transactions for account ");
      desc += QString(job.ref().accountId ().c_str());
   }
    else if (name == "JOBGetStandingOrders"){
      desc = tr("Get all standing orders for account ");
      desc += QString(job.ref().accountId ().c_str());
   }
   else if (name == "JobGetDatedSingleTransfers"){
      desc = tr("Get all dated transfers for account ");
      desc += QString(job.ref().accountId ().c_str());
   }
   else if (name == "JobSingleTransfer"){
      desc = tr("Single transfer from account ");
      desc += QString(job.ref().accountId ().c_str());
    }
    else if (name == "JobDatedSingleTransfer"){
      desc = tr("Dated transfer from account ");
      desc += QString(job.ref().accountId ().c_str());
    }
    else if (name == "JobStartStandingOrder"){
      desc = tr("Standing order from account ");
      desc += QString(job.ref().accountId ().c_str());
    }
   
   
   
   if (desc.isEmpty()) desc = name;
   
   new QListViewItem(listViewAccountJobs, QString::number(_count++) ,desc);
}



/**
*analyzing of the admin job
*@param job pointer on AdminJob
*/
void GuiJobs::analyzeAdminJob(Pointer<AdminJob> job)
{
   QString name = QString(job.ref().name().c_str());    
   QString desc = name;
         
   if (name == "JobGetAccounts"){
      desc = tr("Get a list of all accounts");
      }
   else if (name == "JobSync"){
      	desc = tr("Synchronisation with bank server");
      }
   else if (name == "JobGetKeys"){
      desc = tr("Get public keys of the bank");
      }
   else if (name == "JobSendKeys"){
      desc = tr("Send own public keys to the bank");
      }  
     
   new QListViewItem(listViewAdminJobs, QString::number(_count++) ,desc);
}




/**
*sets the selected account jobs
*/
void GuiJobs::slotSelectAccountJob()
{
     listViewAdminJobs->setSelected(listViewAdminJobs->selectedItem(), false);
     _adminSelected = false;
     _accountSelected = true;    
}

/**
*sets the selected admin jobs
*/
void GuiJobs::slotSelectAdminJob()
{
      listViewAccountJobs->setSelected(listViewAccountJobs->selectedItem(), false);
      _adminSelected = true;
     _accountSelected = false;
}

/**
*slot for deleting jobs
*/
void GuiJobs::slotDeleteJob()
{
   int selected;
   if (_adminSelected){
      
      // prevent null pointer exception
      if (listViewAdminJobs->selectedItem() == NULL) return;
      
      selected = listViewAdminJobs->selectedItem()->text(0).toInt() -1; 
      
      bool keep  = QMessageBox::question(this, 
                                          tr("Delete?"), 
                                          tr("Do you really want to delete the job \"") + 
                                          listViewAdminJobs->selectedItem()->text(1) +
                                          tr("\"\nwith job id:  ") + 
                                          listViewAdminJobs->selectedItem()->text(0) + "?",
                                          tr("&Yes"), tr("&No"), QString::null, 0, 1);
      
       if (!keep) 
       _hobit->deleteAdminJob(selected);  
   }
   else if (_accountSelected){
       
       // prevent null pointer exception
       if (listViewAccountJobs->selectedItem() == NULL) return;
          
       selected = listViewAccountJobs->selectedItem()->text(0).toInt() -1; 
       
       bool keep  = QMessageBox::question(this, 
                                           tr("Delete?"), 
                                           tr("Do you really want to delete the job \"") + 
                                           listViewAccountJobs->selectedItem()->text(1) +
                                           tr("\"\nwith job id:  ") + 
                                           listViewAccountJobs->selectedItem()->text(0) + "?",
                                           tr("&Yes"), tr("&No"), QString::null, 0, 1);
       
        if (!keep) 
        _hobit->deleteAccountJob(selected);   
   }  
   _main->refresh();
}


/**
*sends the jobs when executes
*/
void GuiJobs::slotSendNow()
{
  _main->slotSendNow();
}
/** Deletes all jobs
*/
void GuiJobs::slotDeleteAllJobs(){
   int jobNumber;
   QListViewItem * item = NULL;
   QListView listview;
   bool keep  = QMessageBox::question(this, 
   				       tr("Delete All?"), 
   		 	   	       tr("Do you really want to delete all jobs? <br> \
                                           There is no way to restore them. \""), 
			               tr("&Yes"), tr("&No"), QString::null, 0, 1);
  if(!keep){ //clear account jobs list
   if((item = listViewAccountJobs->firstChild())!= NULL) //empty list    
    do{
       jobNumber = item->text(0).toInt() -1; 
       _hobit->deleteAccountJob(0);  
      } while((item =item->nextSibling()) != NULL);   
  }//end if                                    
 
 if(!keep){//clear admin jobs list
   if((item = listViewAdminJobs->firstChild())!= NULL) //empty list
    
    do{
       jobNumber = item->text(0).toInt() -1; 
       _hobit->deleteAdminJob(0);  
      } while((item = item->nextSibling()) != NULL);
  }//end if                                    
       _main->refresh(); 
}
