/***************************************************************************
                          guimain.cpp  -  description
                             -------------------
    begin                : Do Nov 4 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "guimain.h"
#include "guitools.h"
#include "guiwelcome.h"
#include "guibalances.h"
#include "guitransactions.h"
#include "guigettransactions.h"
#include "guiiniletter.h"
#include "guisingletransfer.h"
#include "guidatedsingletransfer.h"
#include "guidatedtransfers.h"
#include "guistandingorders.h"
#include "guistartstandingorder.h"
#include "guitransactionprinter.h"
#include "guihelpwindow.h"
#include "guiknownbanks.h"
#include "guilogin.h"
#include "guilog.h"
#include "guiaccountinformation.h"
#include "guihbciwizard.h"
#include "guichangeaccountname.h"
#include "guiaddaccount.h"
#include "guijobs.h"
#include "guihardwareconfig.h"
#include "guifindmedium.h"
#include "guiloginpassword.h"
#include "guidataexport.h"
#include "guisqlquery.h"

#include "hobit.h"
#include "hobit_connector.h"
#include "hobit_crypt.h"


#include "keyfileopenhbci.h" //to be removed!!!!


#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qwidget.h>
#include <qmenubar.h>
#include <qstringlist.h>
#include <qcursor.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qinputdialog.h>

/**
*constructor for GuiMain (main window),
*sets statusbar, menulist, refreshes informations, widgets, slots, etc. 
*@param crypt pointer on the HoBIT_Crypt interface
*@param parent pointer on current QWidget
*@param name title of the GuiMain dialog
*/
GuiMain::GuiMain() : GuiMainBase(NULL,"guiMain")
{ 
  qDebug("Constructor GuiMain starts");

  _crypt = new HoBIT_Crypt();
  qDebug("Crypt constructed.");
  //Create main configuration directory
  QString homeDir = getenv("HOME"); //this is on the ramdisk, when running from CD
  //create data dir if does not exist
  QString mpHomeDirName = homeDir + "/" + MONEYPENNY_MAIN_DIR;
  QDir mpHomeDir(mpHomeDirName);
  if(!mpHomeDir.exists()){  
    if(!mpHomeDir.mkdir(mpHomeDirName)){
       QMessageBox::critical(NULL, QObject::tr("Error"), 
                             QObject::tr("Could not create main configuration data directory: ") + mpHomeDirName + QObject::tr("Aborting"));
       qDebug("Could not create main configuration data directory: %s", mpHomeDirName.latin1());
       exit (1);
     }
  } 
  _crypt->setConfigDataDir(mpHomeDirName);
 
  setCaption(QString::fromUtf8("MoneyPenny")+ " " + QString(VERSION));
  textEditHelp->setTextFormat(Qt::RichText);

  _currentWidget = NULL;
  _currentWizardPage = NULL;
  _currentAccount = "";
  _exitCode = EXITCODE_SUCCESS; // normal shutdown - user will be asked
   
   //set all child widgets with NULL
  _guitools = NULL;
  _guiwelcome = NULL;
  //_guibalances = NULL;
  _guiaccountinformation = NULL;
  _guitransactions = NULL;
  _guitransactionprinter = NULL;
  _guigettransactions = NULL;
  _guiiniletter = NULL;
  _guisingletransfer = NULL;
  _guidatedsingletransfer = NULL;
  _guidatedtransfers = NULL;
  _guistandingorders = NULL;
  _guistartstandingorder = NULL;
  _guihelpwindow = NULL;
  _guiknownbanks = NULL;
  _guilogin = NULL;
  _guiNullWidget = NULL;
  _guihbciwizard = NULL;
  _guichangeaccountname = NULL;
  _guijobs = NULL;
  _guiaddaccount = NULL;
  _guihardwareconfig = NULL;
  _guilog = NULL;
  _guitransfertemplates = NULL;
  _guitransactioninfo = NULL;
  _guiburn = NULL;			// delete after CeBIT
  _guiSplash = NULL;
  _guiFindMedium = NULL;
  _guiLoginPassword = NULL;
  _guisqlquery = NULL;
  _guisqlqueryresult = NULL;

   //Initialize splash and findMedium and LoginPassword only. They are needed immediately.
   //All other child widgets are initialized by buildMainApplication while the user looks at the splash.	
   _guiSplash = new GuiSplash();
   qDebug("guiSplash constructed.");
   _guiFindMedium = new GuiFindMedium(_guiSplash);
   qDebug("guiFindMedium constructed.");
   _guiLoginPassword= new GuiLoginPassword(_guiSplash, _crypt);
   qDebug("guiLoginPassword constructed.");
}


void GuiMain::buildMainApplication(){
   qDebug("Start build application");
  _connector = new HoBIT_Connector(this);
  _hobit = new HoBIT( false,
                      this,
                      MONEYPENNY_OPENHBCI_FILE,
                      MONEYPENNY_AQMONEY_FILE,
		      MONEYPENNY_TEMPLATE_FILE,
                      MONEYPENNY_HBCIDATA_DIR,
                      APPLICATION_NAME,
                      _crypt,
                      _connector);
                      
  connect(_hobit, SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));
  connect(_hobit->getAccountJobFactory(), SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));
  connect(_hobit->getAdminJobFactory(), SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));
  _guilog = new GuiLog(widgetStack, "logwidget");
  widgetStack->addWidget(_guilog, LOG_WIDGET);
  connect(_guilog, SIGNAL(closeRequest()), this, SLOT(slotShowLastVisibleWidget()));
  
  _guiNullWidget = new QWidget(widgetStack, "nullwidget");
  widgetStack->addWidget(_guiNullWidget, NULL_WIDGET);

  _guiiniletter = new GuiIniLetter(widgetStack, this, "iniletter", _hobit);
  widgetStack->addWidget(_guiiniletter, INILETTER_WIDGET);
  connect(_guiiniletter, SIGNAL(terminated(int)), this, SLOT(slotRaiseWidget(int)));

  _guitools = new GuiTools(widgetStack, this, "toolbox", _hobit);
  connect(_guitools, SIGNAL(helpRequest(const QString& )), this, SLOT(slotHelp(const QString& )));
  connect(_guitools, SIGNAL(importRDH2Key(const QString & )), this, SLOT(slotImportRDH2Key(const QString & )));
  connect(_guitools, SIGNAL(sqlquery()), this, SLOT(slotSqlQuery()));
  widgetStack->addWidget(_guitools, TOOLBOX_WIDGET);

  _guiwelcome = new GuiWelcome(widgetStack, "welcome");
  widgetStack->addWidget(_guiwelcome, WELCOME_WIDGET);

  qApp->processEvents(); //keep gui alive
  
  //   _guibalances = new GuiBalances(this, widgetStack, "balances", _hobit);
  //  widgetStack->addWidget(_guibalances, BALANCES_WIDGET);

  _guiaccountinformation = new GuiAccountInformation(widgetStack, this, _hobit);
  widgetStack->addWidget(_guiaccountinformation, ACCOUNT_INFORMATION_WIDGET);
  
  _guichangeaccountname = new GuiChangeAccountName(widgetStack, this, "changeAccountName", _hobit);
  widgetStack->addWidget(_guichangeaccountname, CHANGE_ACCOUNT_NAME_WIDGET);

  _guitransactions = new GuiTransactions(widgetStack, this,  "transactions", _hobit);
  widgetStack->addWidget(_guitransactions, TRANSACTIONS_WIDGET);
  connect(_guitransactions, SIGNAL(printTransactions()), this, SLOT(slotPrintTransactions()));
  connect(_guitransactions, SIGNAL( showDetails(Pointer<Transaction>)), this, SLOT(slotShowTransactionInfo(Pointer<Transaction>)));
  connect(_guitransactions, SIGNAL(exportSelected(QTable* )), this, SLOT(slotExport(QTable*)));

  _guitransactioninfo = new GuiTransactionInfo(widgetStack, this, _hobit);
  widgetStack->addWidget(_guitransactioninfo, TRANSACTION_INFORMATION_WIDGET);
  connect(_guitransactioninfo, SIGNAL(goBack()), this, SLOT(slotShowLastVisibleWidget()));
 
  _guigettransactions = new GuiGetTransactions(widgetStack, this, "gettransactions", _hobit);
  widgetStack->addWidget(_guigettransactions, GET_TRANSACTIONS_WIDGET);
  connect(_guigettransactions, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));
  qDebug("GuiGetTransActions constructed");

  _guitransactionprinter = new GuiTransactionPrinter(this, "transactionprinter");
  widgetStack->addWidget(_guitransactionprinter, TRANSACTIONS_PRINTER_WIDGET);
  connect(_guitransactionprinter, SIGNAL(cancelled()), this, SLOT(slotShowTransactions()));

  _guisingletransfer = new GuiSingleTransfer(widgetStack, this, "guisingletransfer", _hobit);
  widgetStack->addWidget(_guisingletransfer, SINGLE_TRANSFER_WIDGET);
  connect(_guisingletransfer, SIGNAL(cancelled()), this, SLOT(slotShowLastVisibleWidget()));

  _guidatedtransfers = new GuiDatedTransfers(widgetStack, this,  "guidatedtransfers", _hobit);
  widgetStack->addWidget(_guidatedtransfers, DATED_TRANSFERS_WIDGET);
  connect(_guidatedtransfers, SIGNAL(raiseWidget(int)), this, SLOT(slotRaiseWidget(int)));

  _guistandingorders = new GuiStandingOrders(this, widgetStack, "guistandingorders", _hobit);
  widgetStack->addWidget(_guistandingorders, STANDING_ORDERS_WIDGET);
   connect(_guistandingorders, SIGNAL(showInformation(Pointer<StandingOrder>)), this, SLOT(slotShowStandingOrderInfo(Pointer<StandingOrder>)));

  _guitransfertemplates = new GuiTransferTemplates(this, widgetStack, "guitransfertemplates", _hobit);
  widgetStack->addWidget(_guitransfertemplates, TRANSFER_TEMPLATES_WIDGET);

  _guidatedsingletransfer = new GuiDatedSingleTransfer(widgetStack, this, "guidatedsingletransfer", _hobit);
  widgetStack->addWidget(_guidatedsingletransfer, DATED_SINGLE_TRANSFER_WIDGET);
  connect(_guidatedsingletransfer, SIGNAL(cancelled()), this, SLOT(slotDatedTransfer()));

  qApp->processEvents(); //keep gui alive
 
  _guichangeaccountname = new GuiChangeAccountName(widgetStack, this, "guichangeaccountname", _hobit);
  widgetStack->addWidget(_guichangeaccountname, CHANGE_ACCOUNT_NAME_WIDGET);

  _guistartstandingorder = new GuiStartStandingOrder(widgetStack, this, "guistartstandingorder", _hobit);
  widgetStack->addWidget(_guistartstandingorder, START_STANDING_ORDER_WIDGET);
  connect(_guistartstandingorder, SIGNAL(cancelled()), this, SLOT(slotStandingOrder()));

  _guihardwareconfig = new GuiHardwareConfig(widgetStack, this, "hardwareconfig");
  widgetStack->addWidget(_guihardwareconfig, HARDWARE_WIDGET);
  qDebug("GuiHardwareConfig constructed");

  _guiknownbanks = new GuiKnownBanks(widgetStack, "guiknownbanks", _hobit);
  widgetStack->addWidget(_guiknownbanks, KNOWNBANKS_WIDGET);
  qDebug("GuiKnownBanks constructed");

  _guijobs = new GuiJobs(widgetStack, this, "jobs", _hobit);
  widgetStack->addWidget(_guijobs, JOBS_WIDGET);
  qDebug("GuiJobs constructed");

  _guiaddaccount = new GuiAddAccount(widgetStack, this, "add account", _hobit);
  widgetStack->addWidget(_guiaddaccount, ADD_ACCOUNT_WIDGET);
  qDebug("GuiAddAccount constructed");

  _guihbciwizard = new GuiHbciWizard(this, "guihbciwizard", _hobit);
  widgetStack->addWidget(_guihbciwizard, HBCI_WIZARD);
  connect(_guihbciwizard, SIGNAL(helpRequest(const QString& )), this, SLOT(slotHelp(const QString& )));
  connect(_guihbciwizard, SIGNAL(setupSuccessful(HBCI_MODE)), this, SLOT(slotSetupSuccessful(HBCI_MODE)));
  connect(_guihbciwizard, SIGNAL(setupInternet(int)), this, SLOT(slotRaiseWidget(int)));
  connect(_guihbciwizard, SIGNAL(showLog(int)), this, SLOT(slotRaiseWidget(int)));
  connect(_guihbciwizard, SIGNAL(testInternet(const QString& , int, ConnectionState * )), 
           this, SLOT(slotTestInternet(const QString& , int , ConnectionState * )));
  connect(_guihbciwizard, SIGNAL(renewHoBIT()), this, SLOT(renewHoBIT()));
  connect(_guihbciwizard, SIGNAL(addAccount()), this, SLOT(slotAddAccount()));
  connect(_guiaddaccount, SIGNAL(accountAdded()), _guihbciwizard, SLOT(findAccounts()));
  qDebug("GuiHbciWizard constructed");
 
  qDebug("Running from: %s", QDir::currentDirPath().latin1());
  QStringList helpDirs;
  helpDirs << "." << "help";
  _guihelpwindow = new GuiHelpWindow(widgetStack, "Helpwindow", MP_DIR() + MONEYPENNY_HELP_DIR + "/index.html", helpDirs);
  widgetStack->addWidget(_guihelpwindow, HELP_WIDGET);
  connect(_guihelpwindow, SIGNAL(cancelled(int)), this, SLOT(slotRaiseWidget(int)));

  qDebug("Creating guiLogin");

  _guilogin = new GuiLogin(widgetStack, this, "Login", _hobit);
  widgetStack->addWidget(_guilogin, LOGIN_WIDGET);
  
 _guisqlquery = new GuiSqlQuery(widgetStack, "SQL-Query");
  widgetStack->addWidget(_guisqlquery, SQL_QUERY_WIDGET);
  connect(_guisqlquery, SIGNAL(sqlQuery(QString )), this, SLOT(slotExecuteSqlQuery(QString )));

 _guisqlqueryresult = new GuiSqlQueryResult(widgetStack, "SQL-Query-Result");
  widgetStack->addWidget(_guisqlqueryresult, SQL_RESULT_WIDGET);
  connect(_guisqlqueryresult, SIGNAL(goback()), this, SLOT(slotShowLastVisibleWidget()));
  
  _guidataexport = new GuiDataExport(widgetStack, "Export");
  widgetStack->addWidget(_guidataexport, EXPORT_WIDGET);
  connect (_guidataexport, SIGNAL(back()), this, SLOT(slotShowLastVisibleWidget()));
  
  // delete after cebit
  if (USE_CEBIT_FEATURES){
  _guiburn = new GuiBurn(widgetStack, "Burn");
   widgetStack->addWidget(_guiburn, BURN_WIDGET);
  }
  
  widgetStack->raiseWidget(LOGIN_WIDGET);
  _currentWidget = widgetStack->visibleWidget(); //save widget to return to

  createMenu();

  //start Timer
  _timer = new QTimer( this );
  connect( _timer, SIGNAL(timeout()), this, SLOT(slotTimerDone()) );
  _timer->start( 1000, FALSE ); // 10 seconds multishot timer

  //show tcp-status
  textLabelConnection->setText(tr("Initialization"));
  displaySmallHelp("smallMain.html");  
}

/**
*destructor of GuiMain
*/
GuiMain::~GuiMain()
{
  delete _guiSplash; //_guiSplash is a top level widget. Needs to be deleted.
  delete _crypt;
  delete _hobit;
  qDebug("Destructor GuiMain finished");
}

/**
*Raises the HBCI-Widget on the WidgetStack 
*/
void GuiMain::slotHBCI()
{
  widgetStack->raiseWidget(HBCI_WIZARD);
  _guihbciwizard->evaluateStoredData();
  textLabelStatus->setText(tr("HBCI-Configuration"));
  displaySmallHelp("smallHBCIconfig.html");
  refresh();
}

/**
*Raises the toolbox widget 
*/
void GuiMain::slotToolbox()
{
  widgetStack->raiseWidget(TOOLBOX_WIDGET);
  textLabelStatus->setText(tr("Toolbox"));
  displaySmallHelp("smallToolbox.html");
  refresh();
}

/**
*Raises the Widget, which shows quick information about all hbci accounts 
*/
void GuiMain::slotAccounts()
{
  widgetStack->raiseWidget(ACCOUNT_INFORMATION_WIDGET);
  textLabelStatus->setText(tr("Account Status"));
  //   _guibalances->updateList();
  _guiaccountinformation->refresh();
  displaySmallHelp("smallAccountInformation.html");
  refresh();
}

/**
*starts execution of the job list 
*/
void GuiMain::slotSendNow()
{
   if(_hobit->areJobQueuesEmpty()){
     QMessageBox::information(this, tr("Information"), tr("Job queue is empty. No jobs to be sent."));
     return;
   }
   
   QString logText= tr("Executing Job Queue");
   QCursor oldCursor = cursor();
   if (_hobit->getHbciMode()== HBCI_MODE_DDV){
    if(!_hobit->isCardReaderOk()){
      QMessageBox::critical(this, tr("Error"), tr("Could not contact security card reader. Is DDV2 plugin installed? Is reader plugged in?"));
      return;
    } 
   }   
   widgetStack->raiseWidget(LOG_WIDGET);
   update();
   setCursor(QCursor(Qt::WaitCursor));
   _guilog->setLogText(logText);
   if (_hobit->executeJobQueue()== HobitErrorNotConnected)
      widgetStack->raiseWidget(WELCOME_WIDGET);
   else {
     _guilog->showResults();   
     refresh();
   }
   setCursor(oldCursor);
}

/**
*Select the accounts for which the transactionlists are to be updated 
*/
void GuiMain::slotGetTransactions()
{
  widgetStack->raiseWidget(GET_TRANSACTIONS_WIDGET);
/*  _hobit->createJobGetBalance();
  _hobit->createJobGetTransactions(Date(), Date(), "*");
  _hobit->createJobGetDatedTransfers(Date(), Date(), "*");
  _hobit->createJobGetStandingOrders("*");
  if (askForSend()) 
    slotSendNow();*/
}


/**
*shows a dialog with the turnovers 
*/
void GuiMain::slotShowTransactions()
{
  //if(!_showTransactionsItem->isSelected())
  //  listViewMenu->setSelected(_showTransactionsItem, true); //synchronize menu
  //else
  //{
    widgetStack->raiseWidget(TRANSACTIONS_WIDGET);
    textLabelStatus->setText(tr("Turnovers"));
    displaySmallHelp("smallTransactions.html");
    refresh();
 // }
}

/**
*creates a single transfer 
*/
void GuiMain::slotSingleTransfer()
{
  widgetStack->raiseWidget(SINGLE_TRANSFER_WIDGET);
  textLabelStatus->setText(tr("Single Transfer"));
  displaySmallHelp("smallSingleTransfer.html");
  refresh();
}

/**
*creates a dated transfer
*/
void GuiMain::slotDatedTransfer()
{
  widgetStack->raiseWidget(DATED_TRANSFERS_WIDGET);
  textLabelStatus->setText(tr("Dated Transfer"));
  displaySmallHelp("smallDatedTransfer.html");
  refresh();
}

/**
*calls a transfer template. This is a user defined mask for frequently used transfers. 
*/
void GuiMain::slotTransferTemplate()
{
  widgetStack->raiseWidget(TRANSFER_TEMPLATES_WIDGET);
  textLabelStatus->setText(tr("Transaction Template"));
  displaySmallHelp("smallTransferTemplate.html");
 }

/**
*creates a standing order
*/
void GuiMain::slotStandingOrder()
{
  widgetStack->raiseWidget(STANDING_ORDERS_WIDGET);
  textLabelStatus->setText(tr("Standing Order"));
  displaySmallHelp("smallStandingOrder.html");
  refresh();
}


/**
*Update slots
*/
void GuiMain::slotUpdate()
{  
  _hobit->createJobGetAccounts();  
  _hobit->createJobGetBalance();
  _hobit->createJobGetTransactions(Date(), Date(), "*");
  _hobit->createJobGetDatedTransfers(Date(), Date(), "*");
  _hobit->createJobGetStandingOrders("*");
  if (askForSend()){ 
    slotSendNow();
  }
}


/**
*Raise widget for undone jobs 
*/
void GuiMain::slotJobs()
{
  widgetStack->raiseWidget(JOBS_WIDGET);
  textLabelStatus->setText(tr("Job-Queue"));
  displaySmallHelp("smallJoblist.html");
  refresh();
}


/**
*refresh informations, lists and widgets
*/
void GuiMain::refresh()
{
  //int id = widgetStack->id(widgetStack->visibleWidget());
  _guiaccountinformation->refresh();
  _guitools->updateList();
  _guistandingorders->refresh();
  _guidatedtransfers->refresh();
  _guitransactions->refresh();
  _guitransactionprinter->refresh();
  _guitransfertemplates->refresh();
  _guitransactioninfo->refresh();
  _guisingletransfer->refresh();
  _guidatedsingletransfer->refresh();
  _guistartstandingorder->refresh();
  _guihbciwizard->evaluateStoredData();
  _guijobs->refresh();
  _guijobs->repaint();
  //widgetStack->raiseWidget(id);


  _accounts = _hobit->listAccounts();
  if (!_accounts.empty())
    _accit = _accounts.begin();
}

/**
*Building Menu by adding items and setting labels, bottom up
*/
void GuiMain::createMenu()
{
    if (USE_CEBIT_FEATURES){
    _cebitItem = new QListViewItem(listViewMenu, tr("CeBIT 2005"));
    _burnItem = new QListViewItem(_cebitItem, tr("Burn Live-Cd"));
  }
  else{
   _cebitItem = NULL;
   _burnItem = NULL;
  }
  
  listViewMenu->setSorting(-1); //switch off sorting
  _exitItem = new QListViewItem(listViewMenu, tr("Exit"));
  
  _helpItem = new QListViewItem(listViewMenu, tr("Help"));
  _knownBanksItem = new QListViewItem(listViewMenu, tr("German Banks"));
  
  _configItem = new QListViewItem(listViewMenu, tr("Setup"));
  _hbciItem = new QListViewItem(_configItem, tr("HBCI-Setup"));
  _ChangeLoginPasswordItem = new QListViewItem(_configItem, tr("Change Login-Password"));
  _ChangeHbciPasswordItem = new QListViewItem(_configItem, tr("Change HBCI-Password"));
  _toolboxItem = new QListViewItem(_configItem, tr("Toolbox"));
  if(USE_LIVE_CD_FEATURES)
   _hardwareItem = new QListViewItem(_configItem, tr("Hardware"));
  else _hardwareItem = NULL;
    
  _jobItem = new QListViewItem(listViewMenu, tr("Jobs"));
  _sendNowItem = new QListViewItem(_jobItem, tr("Execute Job Queue"));
  _jobQueueItem = new QListViewItem(_jobItem, tr("Job-Queue"));
    
  _paymentItem = new QListViewItem(listViewMenu, tr("Payment"));
  _standingOrderItem = new QListViewItem(_paymentItem, tr("Standing Orders"));
  _datedTransferItem = new QListViewItem(_paymentItem, tr("Dated Transfer"));
  _singleTransferItem = new QListViewItem(_paymentItem, tr("Single Transfer"));
  _transferTemplateItem = new QListViewItem(_paymentItem, tr("Transfer Template"));

  _turnOverItem = new QListViewItem(listViewMenu, tr("Turnovers"));
  _exportTransactionsItem = new QListViewItem(_turnOverItem, tr("Export Turnover Data"));
  _printTransactionsItem = new QListViewItem(_turnOverItem, tr("Print Turnovers"));
  _showTransactionsItem = new QListViewItem(_turnOverItem, tr("Show Turnovers"));
  _getTransactionsItem = new QListViewItem(_turnOverItem, tr("Get Turnovers"));
  
  _accountItem= new QListViewItem(listViewMenu, tr("Account Status"));
  _updateAccountList= new QListViewItem(_accountItem, tr("Update Account List")); 
  _selectAccountItem= new QListViewItem(_accountItem, tr("Select an Account"));
  _accountBalanceItem= new QListViewItem(_accountItem, tr("Show Account Status"));
}

/**
*execute slot after pressing an item of the menu
*/
void GuiMain::slotSelectItemFromMenu()
{ 
  if (_accountItem && _accountItem->isSelected()) slotShowWelcome("smallAccounts.html"); 
  else if (_selectAccountItem && _selectAccountItem->isSelected()) showLogin();
  else if (_accountBalanceItem && _accountBalanceItem->isSelected()) slotAccounts();
  else if (_updateAccountList && _updateAccountList->isSelected()) slotUpdateAccountList();
  
  else if (_turnOverItem && _turnOverItem->isSelected()) slotShowWelcome("smallTurnovers.html"); 
  else if (_getTransactionsItem && _getTransactionsItem->isSelected()) slotGetTransactions();
  else if (_showTransactionsItem && _showTransactionsItem->isSelected()) slotShowTransactions();
  else if (_printTransactionsItem && _printTransactionsItem->isSelected()) slotPrintTransactions();
  else if (_exportTransactionsItem && _exportTransactionsItem->isSelected()) slotShowTransactions();
  
  else if (_paymentItem && _paymentItem->isSelected())  slotShowWelcome("smallPayment.html"); 
  else if (_singleTransferItem && _singleTransferItem->isSelected()) slotSingleTransfer();
  else if (_datedTransferItem && _datedTransferItem->isSelected()) slotDatedTransfer();
  else if (_transferTemplateItem && _transferTemplateItem->isSelected()) slotTransferTemplate();
  else if (_standingOrderItem && _standingOrderItem->isSelected()) slotStandingOrder();
   
  else if (_configItem && _configItem->isSelected())  slotShowWelcome("smallConfig.html"); 
  else if (_hbciItem && _hbciItem->isSelected()) slotHBCI();
  else if (_ChangeLoginPasswordItem && _ChangeLoginPasswordItem->isSelected()) slotChangeLoginPassword();
  else if (_ChangeHbciPasswordItem && _ChangeHbciPasswordItem->isSelected()) slotChangeHbciPassword();
  else if (_hardwareItem && _hardwareItem->isSelected()) slotHardwareConfig();
  else if (_toolboxItem && _toolboxItem->isSelected()) slotToolbox();
  
  else if (_jobItem && _jobItem->isSelected())  slotShowWelcome("smallJobs.html"); 
  else if (_jobQueueItem && _jobQueueItem->isSelected()) slotJobs();
  else if (_sendNowItem && _sendNowItem->isSelected()) slotSendNow();
  
  
  else if (_knownBanksItem && _knownBanksItem->isSelected()) slotListKnownBanks();
  
  else if (_helpItem && _helpItem->isSelected()) slotHelp("index.html");
  
  else if (_exitItem && _exitItem->isSelected()) slotClose();
      
  else if (_burnItem && _burnItem->isSelected()) slotRaiseWidget(BURN_WIDGET);
}


/**
*referring to statusbar:
*- get internet connectionState from connector
*- update date and time
*/
void GuiMain::slotTimerDone()
{
  changeBalanceStatus();

  ConnectionState connectionState = _connector->connectionState();
  switch (connectionState){
    case ConnectionUntested  : textLabelConnection->setText(tr("Connection to the internet not yet tested"));break;
    case ConnectionConnecting: textLabelConnection->setText(tr("Trying to connect to the internet"));break;
    case ConnectionFailed    : textLabelConnection->setText(tr("Connection to the internet failed"));break;
    case ConnectionConnected : textLabelConnection->setText(tr("Connected to the Internet"));break;
    case ConnectionClosed    : textLabelConnection->setText(tr("Connection to the internet closed"));break;
    case ConnectionWasOk     : textLabelConnection->setText(tr("Connection to the internet tested successfully"));break;

   default                   : textLabelConnection->setText(tr("Not connected to the internet"));break;
  }
  _currentdate= new QDate();
  _currenttime= new QTime();
  QString date = _currentdate->currentDate().toString();
  QString time = _currenttime->currentTime().toString();
  QString datetime = date + " - " + time;
  textLabelDate->setText(datetime);
}

/**
*refresh account-balance
*/
void GuiMain::changeBalanceStatus()
{
  if (_currentAccount != "")
  {
    Pointer<Account> acc = _hobit->findAccount(_currentAccount.latin1());
    if (acc.isValid())
    {
      QString balance, n;
      balance = acc.ref().bookedBalance().value().toReadableString().c_str();
      QString text = tr("Your Balance: ") + balance + " ";
      textLabelBalances->setText(text);
    }
  }
}

/**
*Prints the transactions.
*/
void GuiMain::slotPrintTransactions()
{
  if (!_printTransactionsItem->isSelected()) //synchronize menu
    listViewMenu->setSelected(_printTransactionsItem, true);
  else
  {
    _guitransactionprinter->setAccountName(getCurrentAccount());
    _guitransactionprinter->setTransactionList(_guitransactions->getTransactionList());
    widgetStack->raiseWidget(TRANSACTIONS_PRINTER_WIDGET);
  }
}

/**
*Call slotHelp (QString&)
*/
void GuiMain::slotHelp()
{
 slotHelp("index.html");
}

/**
*Open the help window
*@param helpfile a helping QString
*/
void GuiMain::slotHelp(const QString & helpfile)
{
  _currentWidget = widgetStack->visibleWidget(); //save widget to return to
  if(_currentWidget == _guihbciwizard)
    _currentWizardPage = _guihbciwizard->currentPage();
  _guihelpwindow->setHelpfile (MP_DIR() + MONEYPENNY_HELP_DIR  + "/" +  helpfile);
  widgetStack->raiseWidget(HELP_WIDGET);
}


/**
*set Menubar and insert accountbalance
*/
void GuiMain::doMenu()
{
  bar = new QMenuBar(this);
  bankingMenu = new QPopupMenu(this);
  bankingMenu->insertItem("&Balance",  this, SLOT(slotAccounts()), CTRL+Key_K );
  bar->insertItem("&Banking", bankingMenu);
}

/**
*Raise widget with list of known banks
*/
void GuiMain::slotListKnownBanks()
{
  widgetStack->raiseWidget(KNOWNBANKS_WIDGET);
  textLabelStatus->setText(tr("german banks"));
  displaySmallHelp("smallKnownBanks.html");
}


/**
*set account to be used from moneypenny after login 
*@param ca QString for the current account
*/
void GuiMain::setCurrentAccount(QString &ca)
{
  _currentAccount = ca;
  slotActivateMenuFunctions();
  refresh();
  slotRaiseWidget(WELCOME_WIDGET);
}


/**
*set account to be used from moneypenny after login 
*/
QString GuiMain::getCurrentAccount()
{
  return _currentAccount;
}


/**
*show wizard for setup hbci
*/
void GuiMain::showHbciWizard()
{
  _guihbciwizard->evaluateStoredData();
  _guihbciwizard->show();
}

/**
*Raise widget for ini-letter
*/
void GuiMain::slotShowIniLetter()
{
  _guiiniletter->newIniLetter();
  widgetStack->raiseWidget(INILETTER_WIDGET);
}

/**
*Switch Account by raise login-widget
*/
void GuiMain::switchAccount()
{
  showLogin();
}

/**
*Change Accoutname
*/
void GuiMain::changeAccountName()
{
  _currentWidget = widgetStack->visibleWidget();
  widgetStack->raiseWidget(CHANGE_ACCOUNT_NAME_WIDGET);
}


/**
*Show last visible widget
*/
void GuiMain::slotShowLastVisibleWidget()
{
  widgetStack->raiseWidget(_currentWidget);
}

/**
*Raise widget for add account
*/
void GuiMain::slotAddAccount()
{
  _currentWidget = widgetStack->visibleWidget();
  widgetStack->raiseWidget(ADD_ACCOUNT_WIDGET);
}

/**
*Raise widget for hardware-configure-dialog
*/
void GuiMain::slotHardwareConfig()
{
  _currentWidget = widgetStack->visibleWidget();
  widgetStack->raiseWidget(HARDWARE_WIDGET);
}


/**
*Raise Login-Dialog and HBCI-Wizard
*/
void GuiMain::showLogin()
{
  
  slotDeactivateMenuFunctions();
  list<Pointer<Account> > accounts = _hobit->listAccounts();
 
  if (accounts.empty()){
         widgetStack->raiseWidget(HBCI_WIZARD);
   }  
  else
  {
    _guilogin->refresh();
    widgetStack->raiseWidget(LOGIN_WIDGET);
  }
  
}

/**
*Close main window after confirming
*/
void GuiMain::slotClose()
{
 close();
}


/**
*Raises widget with given id
*@param id id of the widget which should be raised up
*/
void GuiMain::slotRaiseWidget(int id)
{
  if(id >=0)
  {
    _currentWidget = widgetStack->visibleWidget();
    if ((id == HBCI_WIZARD)&&(_currentWizardPage != NULL))
      _guihbciwizard->showPage(_currentWizardPage);
    widgetStack->raiseWidget(id);
  }
  else slotShowLastVisibleWidget();
}



/**
*function for closing
*/
void GuiMain::closeEvent(QCloseEvent * ce)
{
  if (! _hobit->areJobQueuesEmpty()){
    if (QMessageBox::question( this,
                              tr("MoneyPenny Shutdown"),
                              tr("You have unsent jobs in your job queue. \nDo you really want to close MoneyPenny (Unsent jobs will be lost)?"),
                              tr("&Ok"), tr("&Cancel"),
                              QString::null, 0, 1 )){
	 ce->ignore();
	 return;
   }
  }
  bool no;
  QString sStorage;
  switch (_Storage){
    case STORAGE_FLOPPY  : sStorage = tr("floppy disk in the drive", 
                           "Replacement for sStorage in: please leave your  sStorage  and press Ok.");break;
    case STORAGE_USB     : sStorage = tr("USB stick plugged in", 
                           "Replacement for sStorage in: please leave your  sStorage  and press Ok.");    break;
    case STORAGE_UNKNOWN : sStorage = tr("storage device in place", 
                           "Replacement for sStorage in: please leave your  sStorage  and press Ok."); break;
    default              : sStorage = tr("storage device in place", 
                           "Replacement for sStorage in: please leave your  sStorage  and press Ok."); break;
  }
  if (_exitCode == EXITCODE_SUCCESS){
  no = QMessageBox::question( this,
                              tr("MoneyPenny Shutdown"),
                              tr("If you want to close Money Penny, <br> please leave your ") + sStorage + tr(" and press Ok."),
                              tr("&Ok"), tr("&Cancel"),
                              QString::null, 0, 1 ) ;
  } else no = false;
  if (!no){  
        textLabelStatus->setText(tr("Saving data and configuration"));      
        qApp->processEvents();
        switch (_Storage){
          case STORAGE_FLOPPY  : sStorage = tr("floppy disk", 
                                 "This is the end of the sentence: Could not write to your floppy disk");    break;
          case STORAGE_USB     : sStorage = tr("USB stick", 
                                 "This is the end of the sentence: Could not write to yourUSB stick");       break;
          case STORAGE_UNKNOWN : sStorage = tr("storage device", 
                                 "This is the end of the sentence: Could not write to your storage device"); break;
          default              : sStorage = tr("storage device", 
                                 "This is the end of the sentence: Could not write to your storage device"); break;
        }
        if(!_hobit->saveEnvironment()){
          bool closeAnyway= QMessageBox::question( this,
                              tr("Error"),
                              tr("Could not write to your ") + sStorage + tr(". <br> <br><b>Saving of data failed.</b>"),
                              tr("&Cancel"), tr("&Close Anyway"),
                              QString::null, 0, 1 ) ;
          if(closeAnyway) 
            ce->accept();
          else ce->ignore();
          return;                      
        }   
                           
  	_hobit->removeConfigDataDir();
        if(USE_LIVE_CD_FEATURES){
           QString command = "sudo /usr/sbin/saveconfig-moneypenny " + _crypt->getCryptDir();
           system(command.latin1());
        }
    
        switch (_Storage){
          case STORAGE_FLOPPY  : sStorage = tr("floppy disk", 
                                 "Replacement for sStorage in: Please remove your  sStorage  now and press Ok."); break;
          case STORAGE_USB     : sStorage = tr("USB stick", 
                                 "Replacement for sStorage in: Please remove your  sStorage  now and press Ok."); break;
          case STORAGE_UNKNOWN : sStorage = tr("storage device", 
                                 "Replacement for sStorage in: Please remove your  sStorage  now and press Ok."); break;
          default              : sStorage = tr("storage device", 
                                 "Replacement for sStorage in: Please remove your  sStorage  now and press Ok."); break;
        }
       no= QMessageBox::question( this,
                               tr("MoneyPenny Shutdown"),
                               tr("Data have been encrypted and saved. \nPlease remove your ") + sStorage + tr(" now and press Ok."),
                               tr("&Ok"), tr("&Cancel"),
                               QString::null, 0, 1 ) ;
       if(!no){
        textLabelStatus->setText(tr("Shutting down")); 
        if(USE_LIVE_CD_FEATURES){
           QString command = " sudo umount " + _MountPoint; 
           system(command.latin1());
           qDebug("\nUmounting: %s", command.latin1());
        }//end LIVE_CD 
             
        ce->accept();
       } else ce->ignore(); // cancel was selected after data saving
  }else ce->ignore(); // cancel was selected after first question   
}

/**
*Deactivate Buttons and List
*/
void GuiMain::slotDeactivateMenuFunctions()
{
  buttonAccounts->setEnabled(false);
  buttonTransfer->setEnabled(false);
  buttonTurnover->setEnabled(false);
  buttonUpdate->setEnabled(false);
  buttonSendNow->setEnabled(false);
  //buttonToolbox->setEnabled(false);
  listViewMenu->setEnabled(false); // true for debug
}

/**
*Activate Buttons and List
*/
void GuiMain::slotActivateMenuFunctions()
{
  buttonAccounts->setEnabled(true);
  buttonTransfer->setEnabled(true);
  buttonTurnover->setEnabled(true);
  buttonUpdate->setEnabled(true);
  buttonSendNow->setEnabled(true);
  buttonToolbox->setEnabled(true);
  listViewMenu->setEnabled(true);
  refresh();
}
/** Displays helpFile in the small help widget at the lower left.
*/
void GuiMain::displaySmallHelp(const QString & helpFile){
  QString fileName = MP_DIR()  + MONEYPENNY_HELP_DIR + "/" + helpFile;
  QFile file( fileName); // Read the text from a file
  if ( file.open( IO_ReadOnly ) )
  {
    QTextStream stream( &file );
    textEditHelp->setText( stream.read() );
    file.close();
  }
  else QMessageBox::warning(this, tr("File Error"), tr("Could not open help file : ") 
                                 + fileName + tr("<br>Please check your installation."));
}

/** Read property of HoBIT_Connector m_Connector. 
*/
HoBIT_Connector * GuiMain::getConnector()const{
	return _connector;
}

/**
*shows welcome widget 
*/
void GuiMain::slotShowWelcome(const QString & smallHelpFile){
 displaySmallHelp(smallHelpFile);
 textLabelStatus->setText("Welcome");
 widgetStack->raiseWidget(WELCOME_WIDGET);
}

/** Loads the configuration files. 
*/
int GuiMain::loadEnvironment(){
   _hobit->loadEnvironment(); 
  // get accounts
  _accounts = _hobit->listAccounts();

  // account iterator
  if (!_accounts.empty())
    _accit = _accounts.begin();
  else textLabelBalances->setText(tr("no balances found"));
  _accountCounter = 0;
  
  refresh();
  return 1;
}
/** Adds a log text to the log text widget
*/
void GuiMain::slotAddLogText(QString logText){
 _guilog->addLogText(logText);
}

/** Opens a dialog which asks the user wether the stores jobs should be
* send or should not. If the user answers "Yes" the Job Queue will be
*executed. Otherwise The jobs will be stored for later use.
*/
bool GuiMain::askForSend(){
 bool result = 
  QMessageBox::question
  (  this,
     tr("Send jobs now?"),
     tr("You have added new jobs to the job-queue. Do you want them to be sent now \nor ") +
     tr("do you want to keep them for sending them later?"),
     tr("&Now"), tr("&Later")  ,
     QString::null) == 0 ; 
 if (!result) QMessageBox::information
   ( this,
     tr("Job Queue"),
     tr("New Jobs have been queued. \nPlease do not forget to send them later! ") );
 return result;
}

/** Read property of QString _ConfigDir. 
*/
const QString& GuiMain::getConfigDataDir()const{
  return _crypt->getConfigDataDir();
}

/** Write property of QString _ConfigDataDir. 
*/
void GuiMain::setConfigDataDir( const QString& _newVal){
  _crypt->setConfigDataDir(_newVal);
}  

/** Write property of QString _CryptDir. 
*/
void GuiMain::setCryptDir( const QString& _newVal){
  _crypt->setCryptDir(_newVal);
  _guihbciwizard->setMediumDir(_newVal); //initialize wizard with the same dir. User may modify this later
}  

/** Read property of QString _CryptDir. 
*/
const QString& GuiMain::getCryptDir(){
  return _crypt->getCryptDir();
}  

/**
 *Raises a transfer template
 */
 void GuiMain::slotRaiseTemplate(	int id, 
 					QString name, 
		   		  	QString accountId, 
			  		QString bankId, 
					QString value, 
			  		QString purpose,
					QString purpose2){
 
 if (id != SINGLE_TRANSFER_WIDGET && id != DATED_SINGLE_TRANSFER_WIDGET && id != START_STANDING_ORDER_WIDGET){
 QMessageBox::critical(widgetStack, "problem:", "could not create transfer from template");
 }
 else{
 
 _currentWidget = widgetStack->visibleWidget();
 
 switch (id){
 
 case SINGLE_TRANSFER_WIDGET:  
 
 	_guisingletransfer->lineEditOtherName->setText(name);
	_guisingletransfer->lineEditOtherAccountId->setText(accountId);
	_guisingletransfer->lineEditOtherBankCode->setText(bankId);
	_guisingletransfer->lineEditValue->setText(value);
	_guisingletransfer->lineEditPurpose->setText(purpose);
	_guisingletransfer->lineEditPurpose2->setText(purpose2);
	widgetStack->raiseWidget(SINGLE_TRANSFER_WIDGET);
	break;
 		
 
 case DATED_SINGLE_TRANSFER_WIDGET:
 
  	_guidatedsingletransfer->lineEditOtherName->setText(name);
	_guidatedsingletransfer->lineEditOtherAccountId->setText(accountId);
	_guidatedsingletransfer->lineEditOtherBankCode->setText(bankId);
	_guidatedsingletransfer->lineEditValue->setText(value);
	_guidatedsingletransfer->lineEditPurpose->setText(purpose);
	_guidatedsingletransfer->lineEditPurpose2->setText(purpose2);
	widgetStack->raiseWidget(DATED_SINGLE_TRANSFER_WIDGET);
	break;
 
 
 	break;
 
 case START_STANDING_ORDER_WIDGET:
 
 	_guistartstandingorder->lineEditOtherName->setText(name);
	_guistartstandingorder->lineEditOtherAccountId->setText(accountId);
	_guistartstandingorder->lineEditOtherBankCode->setText(bankId);
	_guistartstandingorder->lineEditValue->setText(value);
	_guistartstandingorder->lineEditPurpose->setText(purpose);
	_guistartstandingorder->lineEditPurpose2->setText(purpose2);
	widgetStack->raiseWidget(START_STANDING_ORDER_WIDGET);
	break;
 
 	break;
 }
 
 
 
 }
 }
 

/** Write property of Storage _Storage. 
*/
void GuiMain::setStorage( const StorageType& _newVal){
   _Storage = _newVal;
   if(_guihbciwizard != NULL)
     _guihbciwizard->setStorageType(_Storage);
}

/** Presently not needed
*/
void GuiMain::showEvent(QShowEvent * se){
}
 
/** Run a test connection to www.google.de or another url
*/
void GuiMain::slotTestInternet(const QString & url, int port, ConnectionState * state){
 
 QString oldUrl = _connector->getUrl();
 int oldPort = _connector ->getPort();
 
 ConnectionState oldState = _connector->connectionState();
 if(oldState == ConnectionConnected) 
   _connector->closeConnection();  
 
 _connector->setUrl(url);
 _connector->setPort(port);
 _connector->connectTCP();
 * state = _connector->connectionState();
 if( * state == ConnectionConnected){
   _connector->closeConnection();
   * state = ConnectionWasOk;  
   QMessageBox::information(this, tr("Internet Connection Test"), 
     tr("Your internet setup seems to be Ok. Could connect to: ") + url + tr(" : Port: " + QString::number(port)));
 }
 else QMessageBox::warning(this, tr("Internet Connection Test"), 
       tr("Your internet setup does not work properly. Connection test failed. <br>") +
       tr("Tried to connect to: ") + url + tr(" : Port: " + QString::number(port) + tr("")));
 
 _connector->setUrl(oldUrl);
 _connector->setPort(oldPort);      
}

/** calls the knoppix restore script.
*/
void GuiMain::restoreOsConfig(){
  if(! USE_LIVE_CD_FEATURES)
    return;
    
  QFile script(getCryptDir() + "knoppix.sh");
  if(script.exists())
    return;
    
  system("sudo " + getCryptDir() + "knoppix.sh");
}

/** Read property of QString _MountPoint. 
*/
const QString& GuiMain::getMountPoint()const{
	return _MountPoint;
}

/** Write property of QString _MountPoint. 
*/
void GuiMain::setMountPoint( const QString& _newVal){
	_MountPoint = _newVal;
}

/** restart the application
*/
void GuiMain::slotRestart(){
   _exitCode = EXITCODE_RESTART;	
   slotClose();
}


/** shows information about a single transaction
*/

void GuiMain::slotShowTransactionInfo(Pointer<Transaction> tr){
   _currentWidget = widgetStack->visibleWidget();
   _guitransactioninfo->showTransaction(tr);
   widgetStack->raiseWidget(TRANSACTION_INFORMATION_WIDGET);
}

/** shows information about a single transaction
*/
void GuiMain::slotShowStandingOrderInfo(Pointer<StandingOrder> sto){
  _currentWidget = widgetStack->visibleWidget(); //save widget to return to 
   _guitransactioninfo->showStandingOrder(sto);
   widgetStack->raiseWidget(TRANSACTION_INFORMATION_WIDGET);
}
/** Import an RDH2 formatted key. 
*/
void GuiMain::slotImportRDH2Key(const QString & fileName){
  QMessageBox::information(this, QString("MoneyPenny"), // ask user for password
                                    QString(tr("This function is not yet implemented completely!")));
/*   bool ok;
 
   QString password=QInputDialog::getText(QString("MoneyPenny"), // ask user for password
                                    QString(tr("Please input the password for the keyfile you want to import!")),
                                    QLineEdit::Password,
                                    0,
                                    &ok);
  if(ok)
   _hobit->importRDH2Key(fileName, password);
  _hobit->importRDH2Key(fileName, "Password");*/ // To be removed!!!!
}


/** Performed after HBCI Wizard has finished
*/
void GuiMain::slotSetupSuccessful(HBCI_MODE mode){
slotActivateMenuFunctions();
if (mode == HBCI_MODE_RDH)
 slotShowIniLetter();
else showLogin();
}


/** shows the splash screen */
  void GuiMain::showSplashScreen(){
    _guiSplash->showFullScreen();
}

/** hides the splash screen */
  void GuiMain::hideSplashScreen(){
  _guiSplash->hide();
}

/** returns the splash screen */
  GuiSplash * GuiMain::getSplashScreen(){
  return _guiSplash;
}

/**Search for the removable medium*/
 bool GuiMain::findMedium(){
  bool mediumOk = _guiFindMedium->exec() == QDialog::Accepted;
  if(mediumOk){
     setMountPoint(_guiFindMedium->getMountPoint());
     setCryptDir(_guiFindMedium->getCryptDirName()); // sets directory in _crypt
     setStorage(_guiFindMedium->getStorage());
     restoreOsConfig();
     return true;
  }
  else {
     QMessageBox::warning(this, QObject::tr("Error"),QObject::tr("Can not use removable medium, aborting."));
     return false;
  }
 }

 /**ask the user for the local data password.*/
void GuiMain::login() {
  _guiLoginPassword->checkForPasswordFile();
  bool passwordOk = _guiLoginPassword->exec() == QDialog::Accepted;
   if(!passwordOk)
  exit (2);
}

/** deletes the old hobit object and creates a new one */
void GuiMain::renewHoBIT(){
  delete _hobit;
  _hobit = new HoBIT( false,
                      this,
                      MONEYPENNY_OPENHBCI_FILE,
                      MONEYPENNY_AQMONEY_FILE,
		      MONEYPENNY_TEMPLATE_FILE,
                      MONEYPENNY_HBCIDATA_DIR,
                      APPLICATION_NAME,
                      _crypt,
                      _connector);

  _guitools->setHoBIT(_hobit);
  _guiaccountinformation->setHoBIT(_hobit);
  _guitransactions->setHoBIT(_hobit);
  _guigettransactions->setHoBIT(_hobit);
  _guiiniletter->setHoBIT(_hobit);
  _guisingletransfer->setHoBIT(_hobit);
  _guidatedsingletransfer->setHoBIT(_hobit);
  _guidatedtransfers->setHoBIT(_hobit);
  _guistandingorders->setHoBIT(_hobit);
  _guistartstandingorder->setHoBIT(_hobit);
  _guiknownbanks->setHoBIT(_hobit);
  _guilogin->setHoBIT(_hobit);
  _guihbciwizard->setHoBIT(_hobit);
  _guichangeaccountname->setHoBIT(_hobit);
  _guijobs->setHoBIT(_hobit);
  _guiaddaccount->setHoBIT(_hobit);
  _guitransfertemplates->setHoBIT(_hobit);
  _guitransactioninfo->setHoBIT(_hobit);

  connect(_hobit, SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));
  connect(_hobit->getAccountJobFactory(), SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));
  connect(_hobit->getAdminJobFactory(), SIGNAL(logTextToShow(QString)), this, SLOT(slotAddLogText(QString)));

  _hobit->loadEnvironment();
  _guihbciwizard->updateAccountList();
  refresh();

}

/** shows data export dialog */
  void GuiMain::slotExport(QTable* table){
   _currentWidget = widgetStack->visibleWidget();
   _guidataexport->setTableToExport(table);
   _guidataexport->setMountPoint(_MountPoint.latin1());

    widgetStack->raiseWidget(EXPORT_WIDGET);
 }
/** Shows a dialog to change the login password.
*/
void GuiMain::slotChangeLoginPassword(){
   if(_guiLoginPassword->checkForPasswordFile()){
     QMessageBox::warning(this, tr("Warning"), tr("Could not read your password verification file."));
   }
   bool passwordOk = _guiLoginPassword->exec() == QDialog::Accepted;
   if(!passwordOk){
     QMessageBox::critical(this, tr("Error"), tr("Wrong Password!"));
   }
   else{
     _guiLoginPassword->switchToEditPasswordMode();
     passwordOk = _guiLoginPassword->exec() == QDialog::Accepted; //changes password in _Crypt
     if(passwordOk){
       if(_hobit->saveEnvironment()){
         QMessageBox::information(this, tr("Information"), tr("Your login password has been changed."));
       } 
       else{                 
         QMessageBox::critical(this, tr("Error"), tr("Could not save your data. Please make shure, that your medium is writeable and try again."));
      }
     }  
   }
}

/** Shows a dialog to change the HBCI password.
*/
void GuiMain::slotChangeHbciPassword(){  
   widgetStack->raiseWidget(HBCI_WIZARD);
   _hobit->saveEnvironment(); //save all data first
   _guihbciwizard->changePassword();
}


/** Shows a dialog to send an SQL-Query
*/
void GuiMain::slotSqlQuery(){  
_currentWidget = widgetStack->visibleWidget();   
widgetStack->raiseWidget(SQL_QUERY_WIDGET);  
}

/** Executes the sql query
*/
void GuiMain::slotExecuteSqlQuery(QString query){  
_currentWidget = widgetStack->visibleWidget();   
widgetStack->raiseWidget(SQL_RESULT_WIDGET);  
}

/** Get a new account list from bank server.
*/
void GuiMain::slotUpdateAccountList(){
    _hobit->createJobGetAccounts();
    if(askForSend())
      slotSendNow();
}
