/***************************************************************************
                          guistandingorders.cpp  -  description
                             -------------------
    begin                : Do Nov 11 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "guistandingorders.h"
#include <qdatetime.h>
#include <qcombobox.h>
#include <qlistview.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>


/**
*constructor of GuiStandingOrder
*
*@param main a pointer on moneypenny´s main class
*@param parent a pointer on the main widgets widget stack
*@param name widgets title
*@param hobit the hobit object, needed for hbci functions
*/
GuiStandingOrders::GuiStandingOrders(GuiMain* main, QWidgetStack *parent, const char *name, HoBIT* hobit )
 : GuiStandingOrdersBase(parent,name) {
   _stack = parent;
   _hobit = hobit;
   _main = main;
   _StandingOrderList.setAutoDelete(true);

   setTabOrder(tableStandingOrders, buttonNewDatedTransfer);
   setTabOrder(buttonNewDatedTransfer, buttonDeleteDatedTransfer);
   setTabOrder(buttonDeleteDatedTransfer, buttonGet);

   refresh();
 
}


/**
*destructor of GuiStandingOrder
*/ 
GuiStandingOrders::~GuiStandingOrders(){
qDebug("Destructor GuiStandingOrders finished");
}




/** 
*updates the listview depending on the selected account 
*/
void GuiStandingOrders::slotChangeAccount(){

    Date fromdate;
    Date todate;

    QDate currentDate;
    currentDate = currentDate.currentDate();

    QString accountId = _main->getCurrentAccount();

    _StandingOrderList.clear();
    setList(_hobit->listStandingOrders(   0, "*", accountId.ascii(), ""));

    update();
}


/** 
*setting the list
*@param list<Pointer<StandingOrder>> pointer on the StandingOrder
*/
bool GuiStandingOrders::setList(list<Pointer<StandingOrder> > transactions){

  int row = 0;
  StandingOrderItem * item;
  
  tableStandingOrders->hide();
  tableStandingOrders->setNumRows(0);
  _StandingOrderList.clear();
  
  list<Pointer<StandingOrder> >::iterator  soIter;

  for (soIter=transactions.begin(); soIter!=transactions.end(); soIter++){
    
    item = new StandingOrderItem(tableStandingOrders, row, *soIter);
    _StandingOrderList.append(item);
    row++;
  }

  tableStandingOrders->adjustColumn(0);
  tableStandingOrders->adjustColumn(2);
  tableStandingOrders->setColumnStretchable(1, true);
  for(row = 0; row <  tableStandingOrders->numRows(); row++)
   tableStandingOrders->adjustRow(row);
  tableStandingOrders->show();
  //update();
  return true;
}


/** 
*function for transmition to the hobit
*/
void GuiStandingOrders::slotGet(){
      _hobit->createJobGetStandingOrders();
      if(_main->askForSend())
       _main->slotSendNow();
}



/** 
*executed when user performs a double click on a list item 
*@param row number of the row
*@param col number of the col
*@param button button_id
*@param mousePos the QPoint of the current mouse position
*/
void GuiStandingOrders::slotSelectItem(int row, int col, int button, const QPoint & mousePos){
  StandingOrderItem* stoi = _StandingOrderList.at(row);
  if (stoi){
  Pointer<StandingOrder> sto = stoi->standingOrder();
  emit showInformation(sto);
 }
}

/** 
*slot for raising up the standing order widget in the widget stack 
*/
void GuiStandingOrders::slotNewStandingOrder(){
    _stack->raiseWidget(START_STANDING_ORDER_WIDGET);
}


/** 
*slot for deleting the standing order  
*/
void GuiStandingOrders::slotDeleteStandingOrder(){

  StandingOrderItem* stoi = _StandingOrderList.at(tableStandingOrders->currentRow());

  if (tableStandingOrders->currentSelection() != -1 && stoi){
  
  Pointer<StandingOrder> sto = stoi->standingOrder();

   string purpose = sto.ref().purpose();

   QString text = "You are about to cancel the following standing order: <br> <br>";
   text += QString(purpose.c_str());
   text += "<br> <br> Are you shure?";


  bool  keep =  QMessageBox::question(  this,
                                        "Cancel order?",
                                        text,
                                       "&Yes", "&No", QString::null, 0, 1 );

  if (!keep){

    _hobit->createJobDeleteStandingOrder(sto);
    _hobit->createJobGetStandingOrders();
    if(_main->askForSend())
     _main->slotSendNow();
   }}
  else QMessageBox::critical(this, tr("problem"), tr("You did not select an item"));

}





/** 
*function for a refresh
*/
void GuiStandingOrders::refresh()
{
  textLabelAccount->setText(_main->getCurrentAccount());
  if (_main->getCurrentAccount() != NULL)
		slotChangeAccount();
}

