/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qsimplerichtext.h>
#include <qwmatrix.h>
#include <qstringlist.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qradiobutton.h>
#include <qprogressdialog.h>
#include <qapplication.h>
#include <qeventloop.h>
#include <qcheckbox.h>
#include <qcolordialog.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qfontdialog.h> 
#include <qlineedit.h>
#include <qdatetime.h>
#include <qmessagebox.h>
 
#include "guitransactionprinter.h"
#include "listviewitems.h"

//#include "guimain.h"

/**
*this is the transaction printer dialog constructor
*
*@param main a pointer on moneypenny´s main class
*@param name widgets title
*/
GuiTransactionPrinter::GuiTransactionPrinter(QWidget* parent, const char *name)
:guiTransactionPrinterBase(parent, name)
{
  _PagesValid = false;
  _PixmapValid = false;
  _PrinterSetupOk = false;
      //Page settings in cm
  _LeftMarginCm =   1.5;
  editLeft->setText(QString::number(_LeftMarginCm, 'f',1));
  _TopMarginCm =    2.5;
  editTop->setText(QString::number(_TopMarginCm, 'f',1));
  _RightMarginCm =  1.5;
  editRight->setText(QString::number(_RightMarginCm, 'f',1));
  _BottomMarginCm = 1.5;
  editBottom->setText(QString::number(_BottomMarginCm, 'f',1));
  _Printer = NULL; //initialize in showEvent or when required
  _PrinterFont = new QFont("Times", 10);
  _PrinterFontMetrics = NULL;
  _PageWidth = 1000;//guessed start values
  _PageHeight = 1300;
  _Scale = 1.0;
  _PreviewPixmap = new QPixmap;
  _Color1 = white;
  _Color2 = QColor(240, 240, 240); // very light gray
  buttonGroupOptions->hide();
}

/**
*this is the transaction printer dialog destructor
*/
GuiTransactionPrinter::~GuiTransactionPrinter()
{
qDebug("Destructor GuiTransactionPrinter called");
// if(_Printer != NULL)
//   delete _Printer;
//  if(_PrinterFont != NULL)
//   delete _PrinterFont; 
//  delete _PreviewPixmap;
qDebug("Destructor GuiTransactionPrinter finished");
}


/** 
* This is a function for creating the html text for printing and preview. Find out text heights by querying
* QSimpleRichText::height after adding 
* respective paragraph of text. Then create a set of preformatted pages with headers and footers. Every page
* is formatted
* as a complete html table. The table rows may be multiline in height, therefore the page height has to be
* tested after 
* every new table row. 
* To find out the total number of pages, pages are preformatted. When all pages are complete, the final text
* is
* created with the headers and footers showing the correct page and total page numbers. 
* Returns number of pages.
* Formatting works as follows: A first guess on the number of lines per page is made according to page and
* font size.
* A test text is built and formatted with that number of lines. If it fits on the page, the length of the
* test text is increased.
* If it is to long, a line is removed and the removing flag is set. Once removing lines has started, the
* page is complete, when
* the text fits into the print rectangle.
*/
int GuiTransactionPrinter::createPrintPages(){
// if(_PagesValid)
//   return _Pages.count();
  QString testText, // the test string for testing the page length
          allPages,
          line, header, footer;
  QStringList allLines; 
  QTime time;
  time.start();       
  uint nPage = 0;
  uint nLine = 0; //for debugging
  uint nPages = 0; // number of all pages
  _Pages.clear(); //delete old pages
  //get the font data for the current printer
  if(_Printer ==NULL)
   initPrinter();//initialize printer and font
  QRect printRect = calculatePrintRect();
  QPainter painter (_Printer);
  painter.setFont(*_PrinterFont);
  if(_PrinterFontMetrics != NULL)
   delete _PrinterFontMetrics;
  _PrinterFontMetrics = new QFontMetrics(painter.fontMetrics()); 
  int fontHeight = _PrinterFontMetrics->height();// for debugging
  int nGuessedLines = (int)((printRect.height() - 5 * fontHeight) / (fontHeight * 1.5)); //take care of header and footer and try 1.5 line pitch
  //Create test header and footer for obtaining exact header and footer heights
  header = createPageHeader(0, 0); //use dummies for page numbers, they are not yet known, no margins
  footer = createPageFooter(0, 0);
 
  if(_TransactionList.count() > 0){
    //create the lines for all pages and put them into allLines
    QString sColor1 = colorToHtml(_Color1);
    QString sColor2 = colorToHtml(_Color2);
    for(TransactionItem * item = _TransactionList.first(); item != NULL; item = _TransactionList.next()){
      nLine ++;
      if(checkBoxLineColor->isChecked() && ((nLine %2) != 0))
      line = item->htmlTableRow(sColor2); //default: lightgray
      else  line = item->htmlTableRow(sColor1);
      allLines += line;
    }
    
    //now create the pages from allLines
    uint nStartLine= 0;
    uint nStopLine = nGuessedLines;
    bool bDocumentFinished = false;
    bool bPageFinished = false;
    bool bRemoving = false;
    QProgressDialog * progDlg = new QProgressDialog(tr("Formatting printer output..."), tr("&Cancel"), 
                                                     ((allLines.count()) / nGuessedLines) + 1, //guessed number of pages
                                                     this, "ProgressDialogCreatePrintPages", true);
    progDlg->setProgress(0); //start the progress dialog 
    
    do{//document loop
      
      do{ //Page loop
          testText = "";
          for(nLine = nStartLine; (nLine < nStopLine) && (nLine < allLines.count()); nLine ++){ //create a test page
            line = allLines[nLine]; //debugging
            testText += line;
          }
          if(textFits(painter, header + testText + footer, printRect.width(), printRect.height())){
            if (bRemoving) //as we are already removing lines, the page is now short enough and ready for use
              bPageFinished = true;
            else nStopLine++; //add the next line, and try again
            if(nStopLine >= allLines.count()){
              bPageFinished = true;
              bDocumentFinished = true;
            }  
          }// end if text fits
          else{ //the page is too long, remove a line
            bRemoving = true;
            nStopLine--;
          }
       }while(!bPageFinished);//end page loop
       
      nPage ++;    
     // do {}while(((time.elapsed() /1000)% 10)!= 0); //time loop for debugging progress dialog
      progDlg-> setProgress(nPage); //set the new value for progress
      qApp->processEvents(); //keep the application responsive, allow progress dialog to repaint progress bar
        
       _Pages += testText; //save the page
       nPages = _Pages.count(); // copy to local variable for debugging
       //settings for next page
       nGuessedLines = nStopLine - nStartLine; //try this for the next page, it might fit better
       nStartLine = nStopLine;
       nStopLine = nStartLine + nGuessedLines;
       if(nStopLine > allLines.count()){
         nStopLine = allLines.count();
        } 
       bRemoving = false;
       bPageFinished = false;      
    }while (!bDocumentFinished);//end document loop 
    
//    delete progDlg; //not needed any longer
    //create final text with correct page numbers 
  }
  else //no Transactions available
    _Pages += tr("No transactions available"); 
  
  nPage = 0;
  for ( QStringList::Iterator itPages = _Pages.begin(); itPages != _Pages.end(); ++itPages ) {
    QString page; //for debugging
    page = createPageHeader(nPage +1, nPages); //page numbers start with 1
    page += *itPages;
    page += createPageFooter(nPage +1, nPages);
    *itPages = page;
    allPages += page + createPageSeparator(); // add a page separator to the long text
    nPage ++;
  } //end for  
    
  textLabelPages->setText(QString::number(nPages));
  spinBoxFrom->setMaxValue(nPages);
  spinBoxFrom->setValue(1);
  spinBoxTo->setMaxValue(nPages);
  spinBoxTo->setValue(nPages);
  _Printer->setMinMax(1, nPages);
  // fclose(debugFile);
  _PagesValid = true;
  //set scroller values
  scrollBarPages->setMaxValue( (int)(_Pages.count() * _PageHeight * 1.1));
  scrollBarPages->setLineStep((int)(_PageHeight * 0.05));
  scrollBarPages->setPageStep((int)(_PageHeight * 1.1));

  _PixmapValid = false;
  
  return nPages;
}

/** 
* This is a functin for sending signal cancelled (connected to gui main, slotTransactions).
*/
void GuiTransactionPrinter::reject(){
  emit cancelled();
}

/** 
* This is a function for printing the text on printer. Uses one simpleRichText object per page.
*/
void GuiTransactionPrinter::slotPrint()
{
    if(_Printer == NULL)
     initPrinter();
    if(!_PrinterSetupOk) 
     setupPrinter();
    if(_PrinterSetupOk){ 
        QPainter painter( _Printer );
        // Check that there is a valid device to print to.
        if ( !painter.device() ) {
          QMessageBox::critical(this, tr("Error"), tr("Printer can not print."));
          return;
        }
        QRect printRect = calculatePrintRect();       
        bool newPageRequired = false; //first page is supplied automatically
        if(_Printer->pageOrder()== QPrinter::FirstPageFirst)
          for (int nPage = _Printer->fromPage(); nPage <= _Printer->toPage(); nPage++){ //page loop forwards
            if(newPageRequired)
              _Printer->newPage(); 
            printPage (painter, _Pages[nPage -1], *_PrinterFont, printRect);   
            newPageRequired = true;
          }//end page loop
         else //LastPageFirst 
          for (int nPage = _Printer->toPage(); nPage >= _Printer->fromPage(); nPage--){ //page loop backwards
            if(newPageRequired)
              _Printer->newPage(); 
            printPage (painter, _Pages[nPage -1], *_PrinterFont, printRect);   
            newPageRequired = true;
          }//end page loop
    }//end if setup ok
}


/** 
* This is a function for creating the page headers.
* @param pageNumber Number of the page
* @param totalPages Amount of pages
*/
QString GuiTransactionPrinter::createPageHeader(int pageNumber, int totalPages){
  QString result;
  QDateTime date = QDateTime(QDate::currentDate());
  QString dateFormat = tr("yyyy/MMM/dd", "Date format for printer output");
 
 result += "<p align =\"center\"><table width = 100% border =0>\n"; //first line is a separate table 
 result += "<tr>";
   result += "<th align = \"left\">";
    result += tr("Account: ") + _AccountName;
   result += "</th>";
   result += "<th align = \"left\">";
    result += tr("Printed: ") + date.toString(dateFormat);
   result += "</th>";
   result += "<th align =\"right\">";
    result+=tr("Page") + " "  + QString::number(pageNumber) + " " + tr("of") + " " + QString::number(totalPages);
   result += "</th>";
 result += "</tr>";
 result += "</table>";
 
 result += "<table width = 99% border =1>\n"; // this defines the central table, 100% width will cause the outer border not to be drawn
 result += "<tr>";
   
   result += "<th width = 10%>";
    result += tr("Date");
   result += "</th>";
   
   result += "<th width = 15%>";
    result += tr("Name");
   result += "</th>";
   
   result += "<th width = 25%>";
    result += tr("Purpose");
   result += "</th>";
   
   result += "<th width = 15%>";
    result += tr("Text");
   result += "</th>";
    
   result += "<th width = 10%>";
    result += tr("Bank Code");
   result += "</th>";
   
   result += "<th width = 10%>";
    result += tr("Account");
   result += "</th>";
   
   result += "<th width = 10% align = \"right\">";
    result += tr("Value");
   result += "</th>";
   
 result += "</tr>\n";
// result += "</table>\n"; 
 
 return result;

}

/** 
* This is a function for creating the page footers.
* @param pageNumber Number of the page
* @param totalPages Amount of pages
*/
QString GuiTransactionPrinter::createPageFooter(int pageNumber, int totalPages){
  QString result;
  QDateTime date = QDateTime(QDate::currentDate());
  QString dateFormat = tr("yyyy/MMM/dd", "Date format for printer output");
  
 result += "</table>\n";  //end of table started by header
 result += "<p align = \"right\">";
 result += QString::number(pageNumber);
 result += "</p>";
 return result;
}

/** 
* Creates a page separator for the print preview.
*/
QString GuiTransactionPrinter::createPageSeparator(){
  return QString("<br><br><hr><br><br>\n");
}

/** 
* Updates the preview. When called for the first time, it creates the preview pixmap and then 
* calls itself again by calling update(). Once the preview pixmap is valid, it is blitted to the
* preview frame according to the scrollbar value.
* @param pe pointer on the QPaintEvent
*/
void GuiTransactionPrinter::paintEvent(QPaintEvent * pe){ 
 if(!_PagesValid){
  createPrintPages();
  update();
 } 
 else{
  if(!_PixmapValid)
    createBackgroundPixmap(); 
  bitBlt(framePreview,                       // dest
         0,                                  //x dest
         0,                                  //y dest
         _PreviewPixmap,                     //source
         0,                                  //x source
         (int)(scrollBarPages->value() * _Scale),    // y source
         _PreviewPixmap->width(),            //width
         framePreview->height(),             //height
         CopyROP);
 }
}

/**  
* Initializes printer and printer font.
*/
void GuiTransactionPrinter::initPrinter(){
  qDebug("GuiTransactionPrinter::initPrinter: Initializing transaction printer.");
  _Printer = new QPrinter( QPrinter::HighResolution );
  _Printer->setFullPage(TRUE);
  _Printer->setOrientation(QPrinter::Landscape);
}

/** 
* Calculates printable area.
*/
QRect GuiTransactionPrinter::calculatePrintRect(){
  if(_Printer == NULL)
    initPrinter();
  QPainter painter( _Printer );
  painter.setFont(*_PrinterFont);
  _PrinterFontHeight = painter.fontMetrics().height();
  QPaintDeviceMetrics metrics ( painter.device());
  _PageWidth = metrics.width();
  _PageHeight = metrics.height();
  int dpiy = metrics.logicalDpiY();
  //calculate page geometry in pixels
  int leftMargin = (int) ( (_LeftMarginCm/2.54)*dpiy ); 
  int topMargin = (int) ( (_TopMarginCm/2.54)*dpiy ); 
  int rightMargin = (int) ( (_RightMarginCm/2.54)*dpiy ); 
  int bottomMargin = (int) ( (_BottomMarginCm/2.54)*dpiy ); 
  return QRect( leftMargin, 
                topMargin, 
                metrics.width() - leftMargin - rightMargin, 
                metrics.height() - topMargin - bottomMargin); //printable area = page without margins
}

/** 
* Activates or deactivates the spin boxes.
*/
void GuiTransactionPrinter::slotRangeChange(){
 spinBoxFrom->setEnabled(radioButtonRange->isChecked());
 spinBoxTo->setEnabled(radioButtonRange->isChecked());
 if(radioButtonAll->isChecked()){
  spinBoxFrom->setValue(1);
  spinBoxTo->setValue(_Pages.count());
 }
}

/** 
* Calls the printer setup dialog.
*/
void GuiTransactionPrinter::slotSetup(){
  if(_Printer == NULL)
    initPrinter();
  setupPrinter();  
}

/** 
* Finds out, whether the text fits into the printable area.
* @param painter QPainter
* @param text QString
* @param pageWidth width of the page
* @param pageHeight height of the page
*/
bool GuiTransactionPrinter::textFits(QPainter & painter, const QString & text, int pageWidth, int pageHeight){
  int presentPageHeight; // for debugging
  QSimpleRichText  * renderer = new QSimpleRichText(text, //render the page
                                *_PrinterFont);
                          /*      textEditTransactions->context(),
                                textEditTransactions->styleSheet(),
                                textEditTransactions->mimeSourceFactory(),
                                pageHeight);*/
  renderer->setWidth( &painter, pageWidth );
  presentPageHeight = renderer->height();
//  delete renderer; //throw it away, it can not be modified
  return presentPageHeight < pageHeight;
}

/** 
* Prints a single page.
* @param painter QPainter
* @param text QString
* @param font QFont
* @param printRect QRect for the printing area
*/
int GuiTransactionPrinter::printPage(QPainter & painter, const QString & text, QFont & font, const QRect & printRect){
  int height;
  QSimpleRichText * richText = new QSimpleRichText( text, font);
  richText->setWidth( &painter, printRect.width() );    
  richText->draw( &painter, printRect.left(), printRect.top(), printRect, colorGroup() );
  height = richText->height();
  delete richText; //throw it away, it can't be modified
  return height;
}

/** 
* Calls QPrinter::setup and synchronizes spin boxes and printer setup dialog.
*/
bool GuiTransactionPrinter::setupPrinter(){
    bool result;
    _Printer->setMinMax(1, _Pages.count());
    if(spinBoxTo->value() < spinBoxFrom->value()){ //user wants reverse order
      _Printer->setPageOrder(QPrinter::LastPageFirst);
      _Printer->setFromTo(spinBoxTo->value(), spinBoxFrom->value()); 
    } 
    else _Printer->setFromTo(spinBoxFrom->value(), spinBoxTo->value()); 
    result = _Printer->setup();
    if(result){ 
      if(_Printer->printRange() == QPrinter::AllPages)
         radioButtonAll->setChecked(true); //this will also enable/disable the spinboxes via slotRangeChange
      else radioButtonRange->setChecked(true);
      if(_Printer->pageOrder() == QPrinter::FirstPageFirst){        
        spinBoxFrom->setValue(_Printer->fromPage());
        spinBoxTo->setValue(_Printer->toPage());
      }
      else{ //LastPage First
        spinBoxFrom->setValue(_Printer->toPage());
        spinBoxTo->setValue(_Printer->fromPage());
      }
      
    }  
    _PrinterSetupOk = result;
    return result;   
}

/** 
* Calls update() to redraw with the new scroller settings.
*/
void GuiTransactionPrinter::slotScrollPages(){
 update();
}

/** 
* Creates the background bitmap for fast print preview.
*/
void GuiTransactionPrinter::createBackgroundPixmap(){
  QRect printRect = calculatePrintRect();
//  printRect.setWidth((int)(printRect.width() * 1.05)); //safety margin for mismatch beetween printer and screen fonts
  int topMargin = printRect.top();
  int shadowSize = (int) (0.015 * _PageWidth);
  QRect pageRect(0,0, _PageWidth, _PageHeight);
  int pageDistance = scrollBarPages->pageStep();
  _Scale = 0.98 *(double) framePreview->width() / (double)_PageWidth; //use the whole frame, but leave 2% for shadow
  int w = (int)(_PageWidth * _Scale) + shadowSize;
  int h = (int)(scrollBarPages->maxValue() * _Scale);
  _PreviewPixmap->resize(w, h);
  _PreviewPixmap->fill(framePreview->paletteBackgroundColor());
  QFont frameFont(*_PrinterFont);
  frameFont.setPixelSize(_PrinterFontHeight);
  QPainter pixmapPainter(_PreviewPixmap);
  QWMatrix m;
  m.scale(_Scale, _Scale);
  m.translate( 2.0 / _Scale, 0);
  pixmapPainter.setWorldMatrix(m);
  for(uint nPage = 0; nPage <  _Pages.count(); nPage ++){
   pageRect.moveTop(nPage * pageDistance);   
   pixmapPainter.setBrush(black); //draw a shadow
   pixmapPainter.drawRect(pageRect.right(), pageRect.top() + shadowSize, shadowSize, pageRect.height());
   pixmapPainter.drawRect(pageRect.left() + shadowSize, pageRect.bottom() , pageRect.width() , shadowSize);
   pixmapPainter.setBrush(white);
   pixmapPainter.drawRect(pageRect);
   printRect.moveTop(pageRect.top() + topMargin);
   //pixmapPainter.setBrush(red); //debugging
   //pixmapPainter.drawRect(printRect);
   int printedHeight = printPage(pixmapPainter, _Pages[nPage], frameFont, printRect);
   if(  (nPage == 0)&& (_Pages.count()>1) 
    && ((printedHeight < (int)(printRect.height() * 0.8))
       ||(printedHeight > (int)(printRect.height() * 1.2))) )
    QMessageBox::warning(this, tr("Font Problem Detected"), tr("This font does not scale properly, there may be problems when printing."));
  }//end for
 _PixmapValid = true;
}

/** 
* Read property of QString m_AccountName. 
*/
const QString& GuiTransactionPrinter::getAccountName()const{
	return _AccountName;
}

/** 
* Write property of QString m_AccountName. 
* @param _newVal QString
*/
void GuiTransactionPrinter::setAccountName( const QString& _newVal){
	_AccountName = _newVal;
}

/** 
* Select line color 1
*/
void GuiTransactionPrinter::slotColor1(){
 QColor newColor = QColorDialog::getColor(_Color1, this);
 if(newColor.isValid()){
   _Color1 = newColor; 
   createPrintPages();
   update();
 }  
}
/** 
* Select line color 2
*/
void GuiTransactionPrinter::slotColor2(){
 QColor newColor = QColorDialog::getColor(_Color2, this);
 if(newColor.isValid()){
   _Color2 = newColor; 
   createPrintPages();
   update();
 }  
}
/** 
* Select printer font
*/
void GuiTransactionPrinter::slotFont(){
  bool modified = false;
  QFont newFont  = QFontDialog::getFont(&modified, *_PrinterFont, this);
  if(modified){
   *_PrinterFont = newFont;
   createPrintPages();
   update();
  }
}
/** 
* Set new page margins
*/
void GuiTransactionPrinter::slotMarginsOk(){
   _LeftMarginCm = editLeft->text().toDouble();
   _RightMarginCm = editRight->text().toDouble();
   _TopMarginCm = editTop->text().toDouble();
   _BottomMarginCm = editBottom->text().toDouble();
   createPrintPages();
   update();
}
/** 
* Send signal help
*/
void GuiTransactionPrinter::slotHelp(){
 emit helpRequest (6);//(  TRANSACTIONS_PRINTER_WIDGET); TODO: replace #6
}
/** 
* Hide or show Options
*/
void GuiTransactionPrinter::slotOptions(){
 if(buttonGroupOptions->isVisible()){
  buttonGroupOptions->hide();
  buttonOptions->setText(tr("Show Print &Options"));
 }
 else{
  buttonGroupOptions->show();
  buttonOptions->setText(tr("Hide Print &Options"));
 }
}

/** 
* Returns a html compatible RGB string.
* @param color QColor
*/
QString  GuiTransactionPrinter::colorToHtml(const QColor & color){
  QString sR, sG, sB;
  sR.setNum(color.red(), 16);
  if(sR =="0") sR= "00";
  sG.setNum(color.green(), 16);
  if (sG=="0") sG= "00"; 
  sB.setNum(color.blue(), 16);
  if(sB == "0") sB = "00";
  return QString("#" + sR+ sG + sB);
}

/** 
* To be called when main data have changed
*/
void GuiTransactionPrinter::refresh(){
 _PagesValid = false;
 _PixmapValid = false;
}

/** 
* Read property of QPtrList<TransactionItem> m_TransactionsList. 
*/
const QPtrList<TransactionItem>& GuiTransactionPrinter::getTransactionList()const{
	return _TransactionList;
}

/** 
* Write property of QPtrList<TransactionItem> m_TransactionsList. 
* @param _newVal QPtrList<TransactionItem>
*/
void GuiTransactionPrinter::setTransactionList( const QPtrList<TransactionItem>& _newVal){
  _TransactionList = _newVal;
  _PagesValid = false;
  _PixmapValid = false;
}

