/***************************************************************************
                          guitransactions.cpp  -  description
                             -------------------
    begin                : Sa Nov 6 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "guitransactions.h"
#include <qcombobox.h>
#include <qdatetime.h>
#include <qlistview.h>
#include <qlineedit.h>
#include <openhbci2/date.h>
#include <openhbci2.h>
#include "aqmoney2/account.h"

#include <qlabel.h>
#include <qpushbutton.h>

/**
*this is the transaction dialog constructor
*
*@param main a pointer on moneypenny´s main class
*@param parent a pointer on the main widgets widget stack
*@param name widgets title
*@param hobit the hobit object, needed for hbci functions
*/
GuiTransactions::GuiTransactions(QWidget *parent, GuiMain *guimain, const char *name, HoBIT* hobit)
: GuiTransactionsBase(parent,name) {
  _hobit = hobit;
  _main = guimain;
  _TransactionList.setAutoDelete(true);
//  tableTransactions->setColumnStretchable(2, true);
//  refresh();  
  qDebug("Constructor GuiTransactions");
}


/**
*this is the transaction dialog destructor
*/
GuiTransactions::~GuiTransactions(){
  qDebug("Destructor GuiTransactions");
}


/**
*this is a function for retrieving the transaction list from the
*hobit object
*
*@param transactions a list of all transactions
*/
bool GuiTransactions::setList(list<Pointer<Transaction> > transactions){

  double income = 0.0;
  double outgo = 0.0;
  int row = 0;
  TransactionItem * item;
  
  tableTransactions->hide();
  tableTransactions->setNumRows(0);
  _TransactionList.clear();
  
  list<Pointer<Transaction> >::iterator  transit;

  for (transit = transactions.begin(); transit != transactions.end(); transit++){
    string value = (*transit).ref().value().toString();
  
    double val = (*transit).ref().value().getValue();
    if (val > 0.0) income += val;
    if (val < 0.0) outgo += val;
    //string pup = (*transit).ref().purpose();
  
    //string text = (*transit).ref().text();
    //Date d = (*transit).ref().valutaDate();
    //QDate date(d.year(), d.month(), d.day());
    item = new TransactionItem(tableTransactions, row, *transit);
    _TransactionList.append(item);
    row++;   
  }

  QString num;
  num.setNum(income,'f', 2);
  lineEditIncome->setText(num + " EUR");
  num.setNum(outgo, 'f', 2);
  lineEditOutGo->setText(num + " EUR");

  tableTransactions->adjustColumn(0);
  tableTransactions->adjustColumn(1);
  tableTransactions->adjustColumn(2);  
  tableTransactions->adjustColumn(3);
  tableTransactions->adjustColumn(4);
  tableTransactions->adjustColumn(5);
  tableTransactions->adjustColumn(6);
  tableTransactions->show();
     
  update();
  return true;
}



/**
*this is a function for updating the listview depending on the selected account
*/
void GuiTransactions::slotChangeAccount(){

    Date fromdate;
    Date todate;

    QDate currentDate;
    currentDate = currentDate.currentDate();
    int dateselection = comboBoxDate->currentItem();

    switch (dateselection){

      case 0:                             fromdate = Date(1,1,currentDate.year());
                                              todate = Date(currentDate.day(), currentDate.month(),currentDate.year());
                                              qDebug("current year");
                                              break;
      case 1:                                 fromdate = Date(1,1,currentDate.year() - 1);
                                              todate = Date(31, 12,currentDate.year()-1);
                                              qDebug("last year");
                                              break;
      case 2:                                 fromdate = Date(1,currentDate.month(),currentDate.year());
                                              todate = Date(currentDate.day(), currentDate.month(),currentDate.year());
                                              qDebug("current month");
                                              break;
      case 3:                                 fromdate = Date(1,currentDate.month()-1,currentDate.year());
                                              todate = Date(31, currentDate.month()-1,currentDate.year());
                                              qDebug("last month");
                                              break;
      case 4:				      if (currentDate.month() < 4){
							fromdate = Date(1,1,currentDate.year());
							todate = Date(31, 3,currentDate.year());
					      }
					      else if (currentDate.month() < 7){
							fromdate = Date(1,4,currentDate.year());
							todate = Date(30, 6,currentDate.year());
					      }
					      else if (currentDate.month() < 10){
							fromdate = Date(1,7,currentDate.year());
							todate = Date(31, 9,currentDate.year());
					      }
					      else if (currentDate.month() <= 12){
							fromdate = Date(1,10,currentDate.year());
							todate = Date(30, 12,currentDate.year());
					      }
                                              qDebug("current quarter");
                                              break;
		case 5:		            if (currentDate.month() < 4){
							fromdate = Date(1,10,currentDate.year() -1);
							todate = Date(30, 12,currentDate.year()-1);
					      }
					      if (currentDate.month() < 7){
							fromdate = Date(1,1,currentDate.year());
							todate = Date(31, 3,currentDate.year());
					      }
					      else if (currentDate.month() < 10){
							fromdate = Date(1,4,currentDate.year());
							todate = Date(30, 6,currentDate.year());
					      }
					      else if (currentDate.month() <= 12){
							fromdate = Date(1,7,currentDate.year());
							todate = Date(31, 9,currentDate.year());
					      }
                                              qDebug("last quarter");
                                              break;
       };

       cerr << fromdate.toString();

    QString accountId = _main->getCurrentAccount();

    setList(_hobit->listTransactions(   0,
                                        "*",
                                        accountId.ascii(),
                                        "*",
                                        fromdate,
                                        todate));

     update();
}



/**
*this is the slot for raising up the transaction widget
*/
void GuiTransactions::slotGetTransactions(){
      _main->slotUpdate();
}



/**
*this is the function which executes when the user performs a double click
*on a list item
*
*@param row number of the row 
*@param col number of the col
*@param button number of the button
*@param mousePos mouse-position
*/
void GuiTransactions::slotSelectItem(int row, int col, int button, const QPoint & mousePos){
  TransactionItem* transit =  _TransactionList.at(row);
 
  if (transit){
  Pointer<Transaction> tr = transit->transaction();
  
  emit showDetails(tr);
}
}


/**
*this is a function for a refresh
*/
void GuiTransactions::refresh()
{
  textLabelAccount->setText(_main->getCurrentAccount());
  if (_main->getCurrentAccount() != NULL)
		slotChangeAccount();
}

/** 
* this is a function for activating GuiTransactionPrinter
*/
void GuiTransactions::slotPrint(){
 emit printTransactions();
}

/** 
* this is a function for reading property of QPtrList<TransactionItem> _TransactionList. 
*/
QPtrList<TransactionItem>  GuiTransactions::getTransactionList()const{
	return _TransactionList;
}

/**
* this is a function for resizing the columns
*
*@param re QResizeEvent
*/
void GuiTransactions::resizeEvent(QResizeEvent * re){
//  if(tableTransactions->columnWidth(2) < 100) //do not make it too narrow
//    tableTransactions->adjustColumn(2);
}

/** Take care of the color for outgo transactions
*/
void GuiTransactions::paintEvent(QPaintEvent * pe){
  for(int row= 0; row < tableTransactions->numRows(); row++){
    QLabel * labelValue = (QLabel *)tableTransactions->cellWidget(row, 2);
    if(labelValue != NULL){
      QString sValue = labelValue->text();
      if(sValue.find("-", 0) >= 0) //value is negative
          labelValue->setPaletteForegroundColor(Qt::red); //outgo
      else labelValue->setPaletteForegroundColor(Qt::black); //income
    }
  }//end for   

}

/** calls export Dialog */
void GuiTransactions::slotExport(){
    emit exportSelected(tableTransactions);
}


