/***************************************************************************
                          guitransactions.h  -  description
                             -------------------
    begin                : Sa Nov 6 2004
    copyright            : (C) 2004 by Marcel Mulch
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GUITRANSACTIONS_H
#define GUITRANSACTIONS_H

#include <qwidget.h>
#include <guitransactionsbase.h>
#include <qwidgetstack.h>
#include <qlayout.h>
#include <qtable.h>

#include "transaction.h"
#include "hobit.h"
#include "guimain.h"
#include "listviewitems.h"


class GuiMain;



/**
* this is the class for showing the transactions
*/
class GuiTransactions : public GuiTransactionsBase
{
  Q_OBJECT

public:

  /**
  *this is the transaction dialog constructor
  *
  *@param main a pointer on moneypenny´s main class
  *@param parent a pointer on the main widgets widget stack
  *@param name widgets title
  *@param hobit the hobit object, needed for hbci functions
  */
  GuiTransactions(QWidget *parent, GuiMain* guimain, const char *name=0, HoBIT* hobit = 0);

  /**
  *this is the transaction dialog destructor
  */
  virtual ~GuiTransactions();

/** set hobit object*/
  void setHoBIT(HoBIT* hobit){
   _hobit = hobit;
  }

  /**
  *this is a function for updating the listview depending on the selected account
  */
  virtual void slotChangeAccount();

  /**
  *this is a function for retrieving the transaction list from the
  *hobit object
  *
  *@param transactions a list of all transactions
  */
  virtual bool setList(list<Pointer<Transaction> > transactions);

  /**
  *this is a function for a refresh
  */
  void refresh();

  /**
  * this is a function for reading property of QPtrList<TransactionItem> _TransactionList. 
  */
  QPtrList<TransactionItem> getTransactionList()const;

  /**
  * this is a function for resizing the columns
  *
  *@param re QResizeEvent
  */
  void resizeEvent(QResizeEvent * re);
  
  /** Take care of the color for outgo transactions
  */
  void paintEvent(QPaintEvent * pe);

private:

  HoBIT* _hobit;
  GuiMain* _main;
  QPtrList<TransactionItem> _TransactionList;

public slots:

  /**
  *this is the slot for raising up the transaction widget
  */
  void slotGetTransactions();

  /**
  *this is the function which executes when the user performs a double click
  *on a list item
  *
  *@param row number of the row 
  *@param col number of the col
  *@param button number of the button
  *@param mousePos mouse-position
  */
  void slotSelectItem(int row, int col, int button, const QPoint & mousePos);

  /**
  * this is a function for activating GuiTransactionPrinter
  */
  void slotPrint();

   /** calls export Dialog */
   void slotExport();

signals:

  void printTransactions();
  void showDetails(Pointer<Transaction>);
  void exportSelected(QTable* table);

};

#endif
