/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guitransferform.h"

#include "globals.h"
#include <qlineedit.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qframe.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qlineedit.h>
#include <qdatetimeedit.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qvalidator.h>
#include <list>
#include <string>
using namespace std;



/**
*constructor of GuiTransferForm
*
*@param main a pointer on moneypenny´s main class
*@param parent a pointer on the main widgets widget stack
*@param name widgets title
*@param hobit the hobit object, needed for hbci functions
*/
GuiTransferForm::GuiTransferForm(QWidget *parent, GuiMain* guimain, const char *name, HoBIT* hobit )
:GuiTransferFormBase(parent,name) {

        _main = guimain;
        _hobit = hobit;
    	refresh();
        _banks = _hobit->listOfAllKnownBanks();
        QRegExp rx("[0-9]{1,9},{1,1}[0-9]{1,2}"); // one to seven digits, a comma and one or two digits
        lineEditValue->setValidator(new QRegExpValidator(rx, lineEditValue, "ValueValidator" ));
        
	// if a lineEdit get changed -> call function for validation
	connect( lineEditOtherName, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputOtherName()) );
	connect( lineEditOtherAccountId, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputOtherAccountId()) );
        connect( lineEditOtherBankCode, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputOtherBankCode()) );
       // connect( lineEditValue, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputValue()) );
        connect( lineEditPurpose, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputPurpose()) );
        connect( lineEditPurpose2, SIGNAL( textChanged(const QString&) ), this, SLOT( validateInputPurpose2()) );

	  setTabOrder(lineEditOtherName, lineEditOtherAccountId);	
 	  setTabOrder(lineEditOtherAccountId,lineEditOtherBankCode);	
   	  setTabOrder(lineEditOtherBankCode, lineEditValue);	
   	  setTabOrder(lineEditValue, lineEditPurpose);	
   	  setTabOrder(dateEditFirst, dateEditLast);
   	  setTabOrder(dateEditLast, comboBoxTurnus);
   	  setTabOrder(comboBoxTurnus, spinBoxDay);
   	  setTabOrder(spinBoxDay, lineEditPurpose);	
	  setTabOrder(lineEditPurpose, lineEditPurpose2);	
   	  setTabOrder(lineEditPurpose2, buttonOk);
 
}

/**
*destructor of GuiTransferForm
*/
GuiTransferForm::~GuiTransferForm(){
	qDebug("Destructor GuiTransferForm finished");
}

/** 
*called function after pressing button of formular,
*gets current inputs of formular for transferjob  
*/
void GuiTransferForm::accept(){
// must be overloaded by child
}

void GuiTransferForm::saveTemplate(){

 // check if all necessary fields are not empty, otherwise errormessage
 if(lineEditOtherName->text() == ' ' || lineEditOtherBankCode->text() == ' ' 
    || lineEditOtherAccountId->text() == ' '    || lineEditValue->text() == ' ')
    QMessageBox::warning( this,
                          tr("Error"),
                          tr("Please fill in all fields. Purpose fields are not mandatory."),
                          tr("&OK"),
                          QString::null, 0, 1 ) ;
 else{ //no error
     // current lineedit-inputs to vars
     const string ourAccount = _main->getCurrentAccount().ascii();   
     const string name = lineEditOtherName->text().ascii();
     const string bankCode = lineEditOtherBankCode->text().ascii();
     const string accountId = lineEditOtherAccountId->text().ascii();
     const string purpose1 =  lineEditPurpose->text().ascii();
     const string purpose2 =  lineEditPurpose2->text().ascii();
     const string value = lineEditValue->text().ascii();
     
     
     
         
     Value val(value, "EUR");    
         
     // text for show all inputs of form, used in messagebox
     QString text =    tr("You are about to create the following transfer template: ") +"\n\n";
     text += tr("From Account:      ") ;
     text +=  ourAccount.c_str();
     text += "\n";
     text += tr("To Account: ");
     text += accountId.c_str();
     text += "\n";
     text += tr("At Bank:              ");
     text += bankCode .c_str();
     text += "\n";
     text += tr("Amount:                 ");
     text += value.c_str();
     text += tr(" EUR")+ "\n\n";
     text += tr("\n\nPurpose:");
     text += purpose1.c_str();
     text += "\n";
     text += purpose2.c_str();
     text += "\n";
     
     // messagebox opens for show input of form and ask for confirm, alternatively decline
     bool no = QMessageBox::question( this,
                                      tr("Save transfer template?"),
                                      text,
                                      tr("&Yes"), tr("&No"),
                                      QString::null, 0, 1) ;
      // after confirm: create and send job 
      if (!no) {
        _hobit->createTransferTemplate(ourAccount,name,accountId, bankCode,val,purpose1,purpose2);
	_main->refresh();
      }//end if yes
  }//end no error
}

/** restores a template */
void GuiTransferForm::fromTemplate(){
	_main->slotRaiseWidget(TRANSFER_TEMPLATES_WIDGET);
}


/**
*Set Current used Account to Label in Formular 
*/
void GuiTransferForm::refresh()
{
	textLabelAccount->setText(_main->getCurrentAccount());
	
	Pointer<Account> account;
        
        if (_main->getCurrentAccount() != "") {
	account = _hobit->findAccount(_main->getCurrentAccount().ascii());
	
	if (account.isValid()){
	     textLabelName->setText(account.ref().userName().c_str());
	}
        else textLabelName->setText("No username found");
	}

	QDate curdate; curdate = curdate.currentDate();
  	dateEditFirst->setDate(curdate);

  	dateEditLast->setDate(QDate(curdate.year(),12,31));

  	spinBoxDay->setMinValue(1);
  	spinBoxDay->setMaxValue(30);
	
}

/** 
*validation of lineEdit name of receiver, 
*opens messagebox if validation is false
*/
void GuiTransferForm::validateInputOtherName(){ 
 
 // set input of lineEdit to var
 QString otherName = HoBIT_String::replaceSpecialChars(lineEditOtherName->text());
 // regular expression allows only letters and blanks with a length of 0 to 35
 QRegExp rx( tr("[a-z,A-Z,ä,ö,ü,Ä,ö,Ü,ß\\s]{0,35}"), "0 to 35 characters legal for a name" );
 QRegExpValidator v( rx, 0 );
 int pos = 0;
 // if validation is true current input get saved to var. var is used for check next time the text get changed
 if(v.validate(otherName, pos)==QValidator::Acceptable  || otherName.isEmpty()) lastInputOtherName = otherName;
 // if validation is false messagebox get opened for show that an invalid input was made
 else{
   QMessageBox::warning( this,
                         tr("Error"),
                         tr("Input Error\n Only letters are allowed here. Not more than 35 letters"),
                         tr("OK"), 
                         QString::null, 0, 1 ) ;
                         }
  lineEditOtherName->setText(lastInputOtherName);
 }

/** 
*validation of lineEdit accountId, 
*opens messagebox if validation is false
*/
void GuiTransferForm::validateInputOtherAccountId()
{
   // set input of lineEdit to var
   QString otherAccountId = lineEditOtherAccountId->text();
  
   // if validation is true current input get saved to var. var is used for check next time the text get changed
   if(_hobit-> validateAccountIdString(otherAccountId)) 
     lastInputOtherAccountId = otherAccountId;
   // if validation is false messagebox get opened for show that an invalid input was made
   else { 
     lineEditOtherAccountId->setText(lastInputOtherAccountId);
     QMessageBox::warning( this,
                         tr("Error"),
                         tr("Input Error\n Only digits are allowed here.\nMaximum is 15 digits."),
                         tr("OK"), 
                         QString::null, 0, 1 ) ;
   }
 }

/** 
*validation of lineEdit bankCode, 
*opens messagebox if validation is false 
*/
void GuiTransferForm::validateInputOtherBankCode()
{
  // set input of lineEdit to var
  QString otherBankCode = lineEditOtherBankCode->text();
   // regular expression allows only digits with a length of 0 to 15
  QRegExp rx( "[0-9]{0,8}" );
  QRegExpValidator v( rx, 0 );
  int pos = 0;
 
  // if validation is true current input get saved to var. var is used for check next time the text get changed
  if(v.validate(otherBankCode, pos)==QValidator::Acceptable  || otherBankCode.isEmpty()) lastInputOtherBankCode = otherBankCode;
  // if validation is false messagebox get opened for show that an invalid input was made
  else { lineEditOtherBankCode->setText(lastInputOtherBankCode);
  QMessageBox::warning( this,
                       tr("Error"),
                       tr("Input Error\n Only digits are allowed here. Exactly 8 digits required."),
                       tr("OK"), 
                       QString::null, 0, 1 ) ;
                       }
 
  //we are ready to check for a bank
  if(otherBankCode.length() == 8){
  int nBank = 0;
    list<HoBIT_Bank>::iterator bit;
    QString bankName = tr("Could not find a bank with the code you input.");
    if (!_banks.empty()){
      bool found = false;
      for (bit = _banks.begin(); (bit != _banks.end()) && (! found); bit++)
      {
        nBank++;
        HoBIT_Bank & bank =*bit;
        if(bank.getBankId()== otherBankCode){
          bankName = bank.getBankName();
          found = true; // bank found, stop it
        }
      }// end for
    }// end if ! empty
    textLabelBankName->setText(bankName);
  }//end if length == 8
}



/** 
*validation of lineEdit Purpose1, 
*opens messagebox if validation is false
*/
void GuiTransferForm::validateInputPurpose()
{
 // set input of lineEdit to var 
 QString purpose = HoBIT_String::replaceSpecialChars(lineEditPurpose->text());
  // regular expression allows letters, blanks and digits with a length of 0 to 20
 QRegExp rx(tr("[a-z,A-Z,0-9,\\!\\?\\+\\-\\/\\*\\_\\#\\<\\>\\(\\)\\:\\.\\,\\s]{0,27}"), "0 to 27 characters legal for a purpose"  );
 QRegExpValidator v( rx, 0 );
 int pos = 0;
 
 // if validation is true current input get saved to var. var is used for check next time the text get changed
 if(v.validate(purpose, pos)==QValidator::Acceptable  || purpose.isEmpty()) lastInputPurpose = purpose;
 // if validation is false messagebox get opened for show that an invalid input was made
 else { 
   QMessageBox::warning( this,
                         tr("Error"),
                         tr("Input Error\n If you need more than 27 characters, please use the next field."),
                         tr("OK"), 
                         QString::null, 0, 1 ) ;
 } 
 lineEditPurpose->setText(lastInputPurpose);                      
}



/** 
*validation of lineEdit Purpose2, 
*opens messagebox if validation is false
*/
void GuiTransferForm::validateInputPurpose2()
{
 // set input of lineEdit to var 
 QString purpose = HoBIT_String::replaceSpecialChars(lineEditPurpose2->text());
  // regular expression allows letters, blanks and digits with a length of 0 to 20
 QRegExp rx(tr("[a-z,A-Z,0-9,\\!\\?\\+\\-\\/\\*\\_\\#\\<\\>\\(\\)\\:\\.\\,\\s]{0,27}"), "0 to 27 characters legal for a purpose"  );
 QRegExpValidator v( rx, 0 );
 int pos = 0;
 
 // if validation is true current input get saved to var. var is used for check next time the text get changed
 if(v.validate(purpose, pos)==QValidator::Acceptable  || purpose.isEmpty()) lastInputPurpose = purpose;
 // if validation is false messagebox get opened for show that an invalid input was made
 else { 
   QMessageBox::warning( this,
                         tr("Error"),
                         tr("Input Error\n If you need more than 27 characters, please use the next field."),
                         tr("OK"), 
                         QString::null, 0, 1 ) ;
 } 
 lineEditPurpose2->setText(lastInputPurpose);                      
}

/**
*called on reject, emit signal cancelled()
*/
void GuiTransferForm::reject(){
 emit cancelled();
}
