/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUITRANSFERFORM_H
#define GUITRANSFERFORM_H

#include <guitransferformbase.h>

#include <qwidget.h>
#include <qwidgetstack.h>
#include "hobit.h"
#include <qwidgetstack.h>
#include <guitransferformbase.h>
#include "guimain.h"

class GuiMain;

/**
* Base class for all transfer formulars
*/
class GuiTransferForm : public GuiTransferFormBase
{
  Q_OBJECT
public:

  /**
  *constructor of GuiSingleTransfer
  *
  *@param main a pointer on moneypenny´s main class
  *@param parent a pointer on the main widgets widget stack
  *@param name widgets title
  *@param hobit the hobit object, needed for hbci functions
  */
  GuiTransferForm(QWidget *parent, GuiMain* guimain, const char *name=0, HoBIT* hobit = 0);

  /**
  *destructor of GuiSingleTransfer
  */
  virtual ~GuiTransferForm();

/** set hobit object*/
  void setHoBIT(HoBIT* hobit){
   _hobit = hobit;
  }

protected:
  HoBIT* _hobit;
  GuiMain* _main;
  // help-vars for input of lineEdits referring to validation
  QString lastInputOtherName;
  QString lastInputOtherAccountId;
  QString lastInputOtherBankCode;
  QString lastInputValue;
  QString lastInputPurpose;
  QString lastInputPurpose2;
  list<HoBIT_Bank> _banks;

signals:
 void cancelled();
   
public slots:

  /**
  *called function after pressing button of formular,
  *gets current inputs of formular for transferjob  
  */
  void accept();
  
  
  /**
  * Saves the current transfer as a transfer template
  */ 
  void saveTemplate();
  
  
  /**
  * Loads a template from the template list
  */ 
  void fromTemplate();
  
  /**
  *called on reject, emit signal cancelled()
  */
  void reject();

  /**
  *Set Current used Account to Label in Formular 
  */
  void refresh();

  /**
  *validation of lineEdit name of receiver, 
  *opens messagebox if validation is false
  */
  virtual void validateInputOtherName();

  /**
  *validation of lineEdit accountId, 
  *opens messagebox if validation is false
  */
  virtual void validateInputOtherAccountId();

  /**
  *validation of lineEdit bankCode, 
  *opens messagebox if validation is false 
  */
  virtual void validateInputOtherBankCode();

  /**
  *validation of lineEdit Purpose1, 
  *opens messagebox if validation is false
  */
  virtual void validateInputPurpose();

/**
  *validation of lineEdit Purpose1, 
  *opens messagebox if validation is false
  */
  virtual void validateInputPurpose2();
};

#endif
