/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Do Nov  4 11:52:24 CET 2004
    copyright            : (C) 2004 by Marcel Mulch, Walter Roth
    email                : marcel@mulch-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapplication.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qtranslator.h>
#include <qdir.h>
#include <qmessagebox.h>

//Styles
#include <qplatinumstyle.h>
#include <qwindowsstyle.h>
#include <qmotifplusstyle.h>

#include "guimain.h"
#include "guiloginpassword.h"
#include "guifindmedium.h"
 

 int main(int argc, char *argv[])
{
  int exitcode = 0;
  setMP_DIR(argv[0]);
  if(USE_LIVE_CD_FEATURES){
    qDebug("USE_LIVE_CD_FEATURES: true");
      // starting chipcard daemon
      // system("sudo run_chipcard2 &");
    }
  else qDebug("USE_LIVE_CD_FEATURES: false");   
  QApplication app(argc, argv);
  qDebug("Application constructed.");
  
  //Set default font size according to screen and teststring size
  uint globalFontSize = 15;
  uint screenWidth = app.desktop()->width();
  if (screenWidth >1500)
    screenWidth = 1500; //not more than 1500, dialogs do not grow
  uint stringWidth; 
  QFont globalFont("helvetica", globalFontSize);  
  QString testString(QObject::tr("abcdefghijklmnopqrstuvwxyz    eeeerrmnncy", 
                         "String with the alphabet for calculating font size. Take care of character probabilty"));
  do{  
    globalFontSize--; 
    QFontMetrics metrics(globalFont);
    stringWidth = metrics.width(testString);
    globalFont.setPointSize(globalFontSize); 
   }while ((stringWidth * 140) / testString.length() > screenWidth ); 
  app.setFont(globalFont); //application wide font size setting for all widgets
  
  // Inintialize translation engine
  QTranslator tor( 0 );
  QString qmFileName =  MP_DIR() + QString(MONEYPENNY_TRANSLATIONS_DIR) + "/moneypenny_" + QTextCodec::locale();
  QFile qmFile(qmFileName);
  if(!qmFile.exists())
   qDebug("Could not find translation file: %s", qmFileName.latin1());
  tor.load( qmFileName );
  app.installTranslator( &tor );
 
 
 // Create some styles for choice
  QPlatinumStyle * style0 = new QPlatinumStyle; 
 // QWindowsStyle * style1 = new QWindowsStyle;
 // QMotifPlusStyle *style2= new QMotifPlusStyle;;
  app.setStyle(style0); // style will be deleted by app on shutdown
 
 
   // create Main Gui
  GuiMain *guiMain = new GuiMain(); 
  qDebug("Main widget constructed.");

    //Show the splash  
  guiMain->showSplashScreen();
  
  guiMain->buildMainApplication(); // Build the main application. This may take some time.
  qDebug("guiMain child widgets constructed.");

  if(guiMain->findMedium()){
    guiMain->login();
    
    //hide the splash screen
    guiMain->hideSplashScreen();
     
    app.setMainWidget(guiMain); // deletes old MainWidget (GuiSplash)
     
    //Select size of the window here
    //guiMain->show();  //use default window size
    //guiMain->showFullScreen(); //use the full screen, this will block the system tray!
     guiMain->showMaximized(); //use full screen, but do not cover system tray
     app.processEvents();
     if( guiMain->loadEnvironment()) //load decrypted data  
     guiMain->showLogin();
     else exit (3);  
    
    exitcode = app.exec();
    qDebug("Event loop terminated");
  }
  else exitcode = 1; //no useable medium
  
  delete guiMain; //deletes all child widgets
  return exitcode;

  //exit codes:
   // 0: Everything was ok
   // 1: No write access to medium
   // 2: Wrong login password
   // 3: Loading environment failed
}


